indexing

	description: "[

		Image moving example.

	]"
	date: "$Date: 2005/10/26 18:07:42 $"
	revision: "$Revision: 1.6 $"

class LOVELY

inherit

	EM_APPLICATION

	EM_KEY_CONSTANTS

	EM_SHARED_BITMAP_FACTORY
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make is
			-- Create the main application.
		do
			-- Initialize `video_subsystem' and `screen'.
			video_subsystem.set_video_surface_width (width)
			video_subsystem.set_video_surface_height (height)
			video_subsystem.set_video_bpp (resolution)
			initialize_screen
			set_window_icon ("icon.png")
			set_window_title ("EiffelMedia lovely")
			set_application_id ("lovely_example")

			-- Initialize and draw whole scene.
			x := 300
			y := 350
			initialize_images
			draw_background
			draw_scene

			-- Event loop.
			create event_loop.make_poll
			event_loop.key_down_event.subscribe (agent handle_key_down_event (?))
			event_loop.quit_event.subscribe (agent handle_quit_event (?))
			event_loop.outside_event.subscribe (agent handle_outside_event)
			event_loop.dispatch
			video_subsystem.disable
		end

feature {NONE} -- Implementation

	draw_background is
			-- Draws the background on the screen.
		do
			draw_image (background_image, 0, 0)
		end

	draw_scene is
			-- Draws the image on the screen.
		do
			draw_image_part (background_image, x-border, y-border, 500 + both_borders,202 + both_borders,x-border, y-border)
			draw_image (image, x, y)
			screen.redraw
		end

	draw_image (an_image: EM_SURFACE; an_x: INTEGER; an_y: INTEGER) is
			-- Draws `an_image' at `an_x' and `an_y'.
		require
			an_image_not_void: an_image /= Void
		do
			screen.blit_surface (an_image, an_x, an_y)
		end

	draw_image_part (a_surface: EM_SURFACE; an_x: INTEGER; an_y: INTEGER; a_width: INTEGER; a_height: INTEGER; dest_x: INTEGER; dest_y: INTEGER) is
			-- Draws `an_image' part from `an_x' and `an_y' to `a_width' and `a_height' on `screen' at `dest_x', `dest_y'.
		require
			a_surface_not_void: a_surface /= Void
		do
			screen.blit_surface_part (a_surface, an_x, an_y, a_width, a_height, dest_x, dest_y)
		end

	initialize_images is
			-- Initialize all images.
		do
			bitmap_factory.create_bitmap_from_image (Image_file_name)
			image := bitmap_factory.last_bitmap
			bitmap_factory.create_bitmap_from_image (Background_file_name)
			background_image := bitmap_factory.last_bitmap
		ensure
			image_not_void: image /= Void
		end

feature -- Event handling

	handle_key_down_event (a_keyboard_event: EM_KEYBOARD_EVENT) is
			-- Handle keyboard events.
		require
			a_keyboard_event_not_void: a_keyboard_event /= Void
		do
			if a_keyboard_event.key = a_keyboard_event.sdlk_escape then
				event_loop.stop
			end
			if a_keyboard_event.key = a_keyboard_event.sdlk_f then
				screen.toggle_fullscreen
			end
		end

	handle_quit_event (a_quit_event: EM_QUIT_EVENT) is
			-- Handle quit events.
		require
			a_quit_event_not_void: a_quit_event /= Void
		do
			event_loop.stop
		end

	handle_outside_event is
			-- Handle the outside event.
		local
			keyboard: EM_KEYBOARD
		do
			create keyboard.make_snapshot
			if keyboard.is_pressed (sdlk_up) then
				y := y - Delta
			end
			if keyboard.is_pressed (sdlk_left) then
				x := x - Delta
			end
			if keyboard.is_pressed (sdlk_right) then
				x := x + Delta
			end
			if keyboard.is_pressed (sdlk_down) then
				y := y + Delta
			end
			draw_scene
		end

feature {NONE} -- Implementation

	image: EM_BITMAP
		-- Foreground image

	background_image: EM_BITMAP
		-- Background image

	x: INTEGER
		-- X position

	y: INTEGER
		-- Y position

	event_loop: EM_EVENT_LOOP
		-- Event loop

	width: INTEGER is 1024
		-- Width of the video surface

	height: INTEGER is 768
		-- Height of video surface

	resolution: INTEGER is 24
		-- Resolution of the video surface

	Delta: INTEGER is 5
		-- Delta indicates how far the surface should
		-- be moved upon an arrow key press

	border: INTEGER is
			-- The width of the clipping border
		do
			Result := 2 * Delta
		end

	both_borders: INTEGER is
			-- The width of both clipping borders
		do
			Result := 2 * border
		end

	Image_file_name: STRING is "lovely.gif"
		-- Name of image we want to display

	Background_file_name: STRING is "matterhorn.jpg"
		-- Name of image we want to display

end