indexing

	description: "[

		Demonstrates how you can have a drawable move across the
		screen at a constant speed. The movement is not influenced by the
		framerate.

	]"
	date: "$Date: 2005/10/26 18:07:42 $"
	revision: "$Revision: 1.7 $"

class NORMALIZED

inherit

	EM_APPLICATION

	EM_TIME_SINGLETON

	SDL_VIDEO_FUNCTIONS_EXTERNAL
		export {NONE} all end

	EM_SHARED_SUBSYSTEMS
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make is
			-- Create the main application.
		do
			-- Initiliase subsystems
			Video_subsystem.set_video_surface_width (Window_width)
			Video_subsystem.set_video_surface_height (Window_height)
			Video_subsystem.set_video_bpp (Screen_resolution)
			Video_subsystem.set_fullscreen (Is_fullscreen_enabled)
			Video_subsystem.enable

			initialize_screen

			-- Check subsystem initialisation
			if Video_subsystem.is_enabled then

				-- Set global options
				set_window_title ("EiffelMedia normalized")
				set_window_icon ("icon.png")
				set_application_id ("normalized")

				-- Setup first scene
				create first_scene.make_scene
				initialize_scene

				-- Set first scene and launch it
				set_scene (first_scene)
				launch

				-- Disable subsystems
				Video_subsystem.disable
			else
				io.error.put_string ("Failed to initialise video subsystem.%N")
			end
		end

feature {NONE} -- Initialization

	initialize_scene is
			-- Create all needed objects
		local
			i, j: INTEGER
		do
			bitmap_factory.create_empty_bitmap (Window_width, Window_height)
			color := bitmap_factory.last_bitmap
			color.lock
			from
				i := 0
			until
				i >= Window_width
			loop
				from
					j := 0
				until
					j >= Window_height
				loop
					color.put_pixel (i, j, create {EM_COLOR}.make_with_rgb(255 - (i*255)//Window_width, 255 - (j*255)//Window_height, 128))
					j := j + 1
				end
				i := i + 1
			end
			color.unlock
			create normalized_speed_string.make_normalized_string
			normalized_speed_string.set_speed (40)
			first_scene.main_container.extend (color)
			first_scene.main_container.extend (normalized_speed_string)

			first_scene.set_frame_counter_visibility (True)

			-- Subscribe event handlers
			first_scene.event_loop.outside_event.subscribe (agent handle_outside_event)
		end

	handle_outside_event is
			-- Process deltas
		local
			delta: INTEGER
		do
			delta := normalized_speed_string.get_delta
			normalized_speed_string.set_x(normalized_speed_string.x + delta)
			normalized_speed_string.set_y(normalized_speed_string.y + delta)
		end

feature {NONE} -- Implementation

	normalized_speed_string: NORMALIZED_SPEED_STRING
		-- String

	color: EM_BITMAP
		-- Color

	first_scene: EM_DRAWABLE_SCENE
		-- Screen

	Window_width: INTEGER is 640
			-- Window width

	Window_height: INTEGER is 480
			-- Window height

	Screen_resolution: INTEGER is 16
			-- Screen resolution

	Is_fullscreen_enabled: BOOLEAN is False
			-- Is fullscreen enabled?

end