indexing

	description: "[
		
		*** Since Eiffel doesn't correctly support threats, this doesn't work yet ***

		A semaphore that can control access to shared ressources.

	]"
	date: "$Date: 2005/10/26 12:19:15 $"
	revision: "$Revision: 1.11 $"

class EM_SEMAPHORE

inherit

	SDL_MUTEX_FUNCTIONS_EXTERNAL
		export {NONE} all end

	DISPOSABLE
		export {NONE} all end

create

	make

feature {NONE} -- Creation

	make(an_initial_value :INTEGER) is
			-- Creates a semaphore with `an_initial_value'.
		do
			-- TODO: How to destroy the semaphore without a finalizer?
			p := sdl_create_semaphore_external (an_initial_value)
		end

feature -- Commands

	wait is
			-- Locks the semaphore and suspends the calling thread if the semaphore value is zero.
			-- This call will automatically decrease the semaphore value.
		local
			error_code : INTEGER
		do
			error_code := sdl_sem_wait_external (p)
			if error_code = -1 then
				print("Error locking the semaphore.")
			end
		end

	try_wait : BOOLEAN is
			-- Attempts to lock the semaphore but does not suspend the calling thread.
			-- Returns True if the semaphore was locked successfully. In this case the
			-- semaphore value was automatically decreased.
			-- Returns False if the semaphore could not be locked or an error occured.
		local
			error_code : INTEGER
		do
			error_code := sdl_sem_try_wait_external (p)
			Result := error_code = 0
		end

	wait_timeout(a_timeout : INTEGER) : BOOLEAN is
			-- Locks the semaphore, but only wait up to a specific maximum time.
			-- `a_timeout' is the maximum wait time in miliseconds.
			-- You should not use this feature if possible, because on some
			-- platforms it is implemented by active waiting (spinning).
			-- Returns True if the lock was successful. In this case the semaphore
			-- value was automatically decreased.
			-- Returns False if a timeout or error occured.
		local
			error_code : INTEGER
		do
			error_code := sdl_sem_wait_timeout_external (p, a_timeout)
			Result := error_code = 0
		end

	unlock is
			-- Releases the lock on the semaphore and increments the semaphore value.
		local
			error_code:INTEGER
		do
			error_code := sdl_sem_post_external (p)
			if error_code/=0 then
				print("Error unlocking the semaphore")
			end
		end

	unlock_if_locked is
			-- Releases the lock on the semaphore only if its value is currently 0.
		do
			mutex.lock
			if value = 0 then
				unlock
			end
			mutex.unlock
		end

feature -- Information

	value : INTEGER is
			-- Returns the current value of the semaphore.
		do
			Result := sdl_sem_value_external (p)
		end

feature {NONE} -- Implementation

		p: POINTER
			-- Pointer to the C semaphore

		mutex : EM_MUTEX is
				-- Mutex to prevent race conditions.
			indexing
				once_status: global
			once
				create Result.make
			end

feature {NONE} -- Implementation

	dispose is
			-- Free external resources.
		do
			sdl_destroy_semaphore_external (p)
		end
end