indexing

	description: "[

		*** Since Eiffel doesn't correctly support threats, this doesn't work yet ***

		Use this class to start a function in another thread.
		The function must not have any open arguments and it has to return
		an integer value.
		
		If someone has a great idea how to implement open arguments: feel free :)
		We would have to create a C pointer to an arbitrary count of arguments
		(ie a TUPLE) that would be passed to the `start' feature.

	]"
	date: "$Date: 2005/10/26 17:25:41 $"
	revision: "$Revision: 1.12 $"

deferred class

	EM_THREAD

inherit

	EM_SHARED_SUBSYSTEMS

	SDL_THREAD_FUNCTIONS_EXTERNAL
			export {NONE} all end

	SDL_FUNCTIONS
			export {NONE} all end

feature -- Control

	start is
			-- Starts the thread.
		require
			not_running : not is_running
		do
			return_code := -1
			is_finished := false

			-- ensure that only one thread is created at a time
			-- otherwise we can't guarantee that the callback function goes to the right place by
			-- avoiding callback override by an other thread creation
			thread_subsystem.thread_creation.wait

			thread_subsystem.receiver.set_function (agent entry_run)
			c_thread_pointer := sdl_create_thread_external (thread_subsystem.dispatcher.c_dispatcher, default_pointer)

			is_running := true
		end

	wait is
			-- Wait for a thread to finish (no timeouts are supported -> worst case: wait forever, deadlock)
			-- This should only be called by an other thread than this one
		require
			running : is_running
		local
			status: POINTER
		do
			sdl_wait_thread_external (c_thread_pointer, status)
		end

	kill is
			-- Aborts the thread gracelessly.
			-- You should avoid using this.
		require
			running: is_running
		do
			sdl_kill_thread_external (c_thread_pointer)
			is_running := false
		ensure
			not_running: not is_running
		end

feature -- Run

	run : INTEGER is
			-- this will be executed
			deferred
			end

feature -- Status information

	is_running: BOOLEAN
			-- Returns True if the thread is running.

	is_finished: BOOLEAN
			-- Return True if the thread has finished its execution.

	id: INTEGER is
			-- Returns the ID of this thread.
		require
			thread_running: is_running
		do
			Result := sdl_get_thread_id_external (c_thread_pointer)
		end

	return_code : INTEGER
			-- The return code of the function that was executed.
			-- This will be -1 if `is_finished' is False.

feature {NONE} -- Implementation

	entry_run: INTEGER is
			-- needed to release the lock
		do
			-- Now we can release the lock and another thread can be safely created
			thread_subsystem.thread_creation.unlock

			-- im prinzip kann der thread nun irgendwas zurckgeben... also nicht nur integer,
			-- muesste das einfach irgendwie schlau machen + schuetzen mit mutex
			return_code := run
			is_finished := true
			Result := return_code
		end

	c_thread_pointer: POINTER
			-- C pointer to the thread struct.

invariant
	--	thread_subsystem_enabled: thread_subsystem.is_enabled

end