indexing

	description: "[

		Eiffel documentation generator.
		TODO: refactor 'generate_library and 'generate_system'
		TODO: refactor library mounting

	]"
	date: "$Date$"
	revision: "$Revision$"

class EDOC

inherit

	EDOC_SHARED_CONTEXT
		export {NONE} all end

	KL_SHARED_ARGUMENTS
		export {NONE} all end

	KL_SHARED_EXCEPTIONS
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

create

	make

feature {NONE} -- Initialization

	make is
			-- Execute 'eiffeldoc'.
		do
			Arguments.set_program_name ("eiffeldoc")

--			if has_character_option ('v') then
--				error_handler.set_info_standard
--				error_handler.report_info_message ("Eiffeldoc in verbose mode%N")
--			end
--
--			if has_character_option ('h') or has_character_option ('?') then
--				report_usage_message (0)
--			end
--
--			select_eiffeldoc_directory
		--	apply_default_options
		--	select_config_file
		--	apply_config_file_options
		--	apply_command_line_options

		--	print_options

--			Exceptions.die (0)

			read_command_line

			if is_system then
				generate_system
			else
				generate_library
			end
		end

	select_eiffeldoc_directory is
			-- Select eiffeldoc directory.
		local
			a_directory: STRING
		do
			error_handler.report_info_message ("Searching eiffeldoc directory")
			-- 1. check environment variable
			a_directory := Execution_environment.variable_value ("EIFFELDOC")
			if a_directory = Void then
				error_handler.report_info_message ("No environment variable 'EIFFELDOC' found. Checking current directory.")
			else
				if not not File_system.directory_exists (File_system.pathname (a_directory, "options")) then
					a_directory := Void
					error_handler.report_info_message ("No 'options' directory found in 'EIFFELDOC' directory. Checking current directory.")
				end
			end
			if a_directory = Void then
				-- 2. check working directory
				a_directory := File_system.absolute_pathname (File_system.current_working_directory)
				if not File_system.directory_exists (File_system.pathname (a_directory, "options")) then
					error_handler.report_info_message ("No 'options' directory found in current directory. Checking Execution command.")
					-- 3. check execution command
					a_directory := File_system.dirname (File_system.absolute_pathname (Arguments.argument (0)))
					if not File_system.directory_exists (File_system.pathname (a_directory, "options")) then
						error_handler.report_info_message ("No 'options' directory found in '"+a_directory+"'.")
						error_handler.report_error_message ("Eiffeldoc directory not found.")
						Exceptions.die (1)
					end
				end
			end
			eiffeldoc_directory := a_directory
			error_handler.report_info_message ("Directory '"+a_directory+"' selected%N")
		ensure
			eiffeldoc_directory_set: eiffeldoc_directory /= Void
		end

	read_command_line is
			-- Read command line arguments.
		local
			value: STRING
			output_directory_name: STRING
			options_file: EDOC_OPTION_FILE
		do
			-- h / ?
			if has_character_option ('h') or has_character_option ('?') then
				report_usage_message (0)
			end

			if Arguments.argument_count > 1 then
				options_filename := execution_environment.interpreted_string (Arguments.argument (1))
				if file_system.file_exists (options_filename) then
					create options_file.make_from_filename (options_filename)
					options_file.apply_options
				else
					options_filename := Void
				end
			end

			-- v / verbose
			if has_option ('v', "verbose") then
				error_handler.set_info_standard
				error_handler.report_info_message ("edoc in verbose mode")
			end

			-- l / library / s / system
			xace_filename := separate_option ('s', "system")
			if xace_filename /= Void then
				is_system := True
				error_handler.report_info_message ("System '"+xace_filename+"' selected")
			else
				xace_filename := separate_option ('l', "library")
				if xace_filename /= Void then
					is_library := True
					error_handler.report_info_message ("Library '"+xace_filename+"' selected")
				end
			end
			if not is_system and not is_library then
				error_handler.report_error_message ("%NNo system or library specified%N")
				report_usage_message (1)
			end

			-- o / output
			output_directory_name := separate_option ('o', "output")
			if output_directory_name = Void then
				output_directory_name := File_system.current_working_directory
			end
			create output_directory.make (Execution_environment.interpreted_string (output_directory_name))
			if not output_directory.exists then
				output_directory.recursive_create_directory
			end
			error_handler.report_info_message ("Output directory is "+output_directory.name)

			-- i / include
			value := separate_option ('i', "include")
			included_clusters := seperate_string (value, ',')
			if included_clusters.count = 0 then
				error_handler.report_info_message ("No include option: All clusters selected")
			end

			-- e / exclude
			value := separate_option ('e', "exclude")
			excluded_clusters := seperate_string (value, ',')

			-- m / mount
			value := separate_option ('m', "mount")
			mounted_library_filenames := seperate_string (value, ',')

			-- t / title
			value := separate_option ('t', "title")
			if value /= Void then
				context.set_documentation_title (value)
				context.set_short_documentation_title (value)
				error_handler.report_info_message ("Title set to '"+value+"'")
			end
			value := separate_option ('u', "title_short")
			if value /= Void then
				context.set_short_documentation_title (value)
				error_handler.report_info_message ("Short title set to '"+value+"'")
			end
		end

	has_character_option (a_character: CHARACTER): BOOLEAN is
			-- Is option 'a_charater' set?
		do
			Result := Arguments.index_of_character_option (a_character) > 0
		end

	has_option (a_character: CHARACTER; a_string: STRING): BOOLEAN is
			-- Is option 'a_character' or 'a_string' set?
		do
			Result := Arguments.index_of_character_option (a_character) > 0 or Arguments.index_of_word_option (a_string) > 0
		end

	separate_option (a_character: CHARACTER; a_string: STRING): STRING is
			-- Argument option of either 'a_character' or 'a_string'.
		do
			Result := Arguments.separate_character_option_value (a_character)
			if Result = Void then
				Result := Arguments.separate_word_option_value (a_string)
			end
		end

feature -- Access

	eiffeldoc_directory: STRING
			-- Eiffeldoc directory

	options_filename: STRING
			-- Options filename

	xace_filename: STRING
			-- XAce filename

	mounted_library_filenames: DS_LIST [STRING]
			-- Mounted library filenames

	output_directory: KL_DIRECTORY
			-- Output directory

	included_clusters: DS_ARRAYED_LIST [STRING]
			-- Included clusters

	excluded_clusters: DS_ARRAYED_LIST [STRING]
			-- Excluded clusters

	a_universe: ET_UNIVERSE
			-- Universe

	variables: DS_HASH_TABLE [STRING, STRING] is
			-- Necessary environment variables (Is this an error in ET_XACE_PARSER since it doesn't check environment variables?)
		local
			value: STRING
		once
			create Result.make_default
			value := execution_environment.variable_value ("GOBO_EIFFEL")
			if value /= Void then
				variables.put (value, "GOBO_EIFFEL")
			end
			value := execution_environment.variable_value ("GOBO_DOTNET")
			if value /= Void then
				variables.put (value, "GOBO_DOTNET")
			end
			value := execution_environment.variable_value ("GOBO_EIFFELBASE")
			if value /= Void then
				variables.put (value, "GOBO_EIFFELBASE")
			end
			value := execution_environment.variable_value ("ISE_EIFFEL")
			if value /= Void then
				variables.put (value, "ISE_EIFFEL")
			end
		end

feature -- Status

	verbose: BOOLEAN
			-- Is verbose mode activated?

	is_library: BOOLEAN
			-- Is a library being parsed?

	is_system: BOOLEAN
			-- Is a system being parsed?

feature -- Basic operations

	generate_library is
			-- Generate universe from library.
		local
			library_error_handler: ET_XACE_DEFAULT_ERROR_HANDLER
			parser: ET_XACE_LIBRARY_PARSER
			file: KL_TEXT_INPUT_FILE
		do
			create file.make (execution_environment.interpreted_string (xace_filename))
			file.open_read
			if file.is_open_read then
				create library_error_handler.make_standard
				create parser.make_with_variables (variables, library_error_handler)
				error_handler.report_info_message ("Parsing xace file")
				parser.set_shallow (False)
				parser.parse_file (file)
				if library_error_handler.has_error then
					error_handler.report_error_message ("Error in xace file")
					Exceptions.die (1)
				end
				error_handler.report_info_message ("Parsing done ("+parser.last_library.clusters.clusters.count.out+" toplevel clusters / "+parser.last_library.clusters.count.out+" total)")
				if included_clusters.count = 0 then
					from
						parser.last_library.clusters.clusters.start
					until
						parser.last_library.clusters.clusters.after
					loop
						included_clusters.force_last (parser.last_library.clusters.clusters.item_for_iteration.name)
						parser.last_library.clusters.clusters.forth
					end
				end

				process_clusters (parser.last_library.clusters)
			else
				error_handler.report_error_message ("Unable to open xace file '"+execution_environment.interpreted_string (xace_filename)+"'")
				Exceptions.die (1)
			end
		end

	generate_system is
			-- Generate universe from system.
		local
			system_error_handler: ET_XACE_DEFAULT_ERROR_HANDLER
			parser: ET_XACE_SYSTEM_PARSER
			file: KL_TEXT_INPUT_FILE
		do
			create file.make (execution_environment.interpreted_string (xace_filename))
			file.open_read
			if file.is_open_read then
				create system_error_handler.make_standard
				create parser.make (system_error_handler)
				error_handler.report_info_message ("Parsing xace file")
				parser.parse_file (file)
				if system_error_handler.has_error then
					error_handler.report_error_message ("Error in xace file")
					Exceptions.die (1)
				end
				error_handler.report_info_message ("Parsing done ("+parser.last_system.clusters.clusters.count.out+" toplevel clusters / "+parser.last_system.clusters.count.out+" total)")
				if included_clusters.count = 0 then
					from
						parser.last_system.clusters.clusters.start
					until
						parser.last_system.clusters.clusters.after
					loop
						included_clusters.force_last (parser.last_system.clusters.clusters.item_for_iteration.name)
						parser.last_system.clusters.clusters.forth
					end
				end

				process_clusters (parser.last_system.clusters)
			else
				error_handler.report_error_message ("Unable to open xace file '"+execution_environment.interpreted_string (xace_filename)+"'")
				Exceptions.die (1)
			end
		end

	process_clusters (a_clusters: ET_CLUSTERS) is
			-- Process 'a_clusters' and generate documentation.
		local
			decorated_ast_factory: ET_DECORATED_AST_FACTORY
			mounted_clusters: ET_CLUSTERS
		do
			-- Load mounted libraries
			from
				mounted_library_filenames.start
			until
				mounted_library_filenames.after
			loop
				mounted_clusters := mount_library (mounted_library_filenames.item_for_iteration)
				if mounted_clusters /= Void then
					from
						mounted_clusters.clusters.start
					until
						mounted_clusters.clusters.after
					loop
						if a_clusters.cluster_by_name (mounted_clusters.clusters.item_for_iteration.name) = Void then
							a_clusters.clusters.force_last (mounted_clusters.clusters.item_for_iteration)
--							excluded_clusters.force_last (mounted_clusters.clusters.item_for_iteration.name)
						end
						mounted_clusters.clusters.forth
					end
				end
				mounted_library_filenames.forth
			end

			create a_universe.make (a_clusters, create {ET_ERROR_HANDLER}.make_standard)

			create decorated_ast_factory.make
			decorated_ast_factory.set_keep_all_breaks (True)
			a_universe.set_ast_factory (decorated_ast_factory)
			a_universe.set_error_handler (error_handler)
			a_universe.set_use_assign_keyword (True)
			a_universe.set_use_attribute_keyword (False)
			a_universe.set_use_convert_keyword (True)
			a_universe.set_use_recast_keyword (False)
			a_universe.set_use_reference_keyword (True)
			a_universe.set_use_void_keyword (True)
			a_universe.activate_processors

			context.set_output_directory (output_directory)
			context.included_clusters.append_last (included_clusters)
			context.ignored_clusters.append_last (excluded_clusters)

			context.set_universe (a_universe)
			context.generate_documentation
		end

	mount_library (a_filename: STRING): ET_CLUSTERS is
			-- TODO
		local
			library_error_handler: ET_XACE_DEFAULT_ERROR_HANDLER
			parser: ET_XACE_LIBRARY_PARSER
			file: KL_TEXT_INPUT_FILE
		do
			create file.make (execution_environment.interpreted_string (a_filename))
			file.open_read
			if file.is_open_read then
				create library_error_handler.make_standard
				create parser.make_with_variables (variables, library_error_handler)
				error_handler.report_info_message ("Parsing mounted library: "+execution_environment.interpreted_string (a_filename))
				parser.set_shallow (False)
				parser.parse_file (file)
				if library_error_handler.has_error then
					error_handler.report_error_message ("Error in xace file")
				else
					error_handler.report_info_message ("Parsing done ("+parser.last_library.clusters.clusters.count.out+" toplevel clusters / "+parser.last_library.clusters.count.out+" total)")
					Result := parser.last_library.clusters
				end
			else
				error_handler.report_error_message ("Unable to open xace file '"+execution_environment.interpreted_string (a_filename)+"' to mount library")
			end
		end

feature {NONE} -- Error handling

	report_usage_message (exit_status: INTEGER) is
			-- Report usage message and exit.
		do
			error_handler.report_error (Usage_message)
			Exceptions.die (exit_status)
		end

	Usage_message: UT_USAGE_MESSAGE is
			-- Geant usage message
		local
			s: STRING
		once
			create s.make_from_string ("[-options]%N")
			s.append_string ("%N")
			s.append_string ("where options include:%N")
			s.append_string ("  -v --verbose : Turn on verbose output%N")
			s.append_string ("  -? -h help : Print this help message%N")
			s.append_string ("XAce selection (exactly one library or system / multiple mounts possible):%N")
			s.append_string ("  -l --library {library.xace} : XAce to generate documentation of a library%N")
			s.append_string ("  -s --system {system.xace} : XAce to generate documentation of a system%N")
			s.append_string ("  -m --mount {xace-list} : Comma-separated list of XAce files which are mounted but no documentation is generated%N")
			s.append_string ("Cluster selection (if 'include' is not specified, all toplevel clusters are selected)%N")
			s.append_string ("  -i --include {cluster-list} : Comma-separated list of toplevel clusters to include%N")
			s.append_string ("  -e --exclude {cluster-list} : Comma-separated list of clusters to exclude (all subclusters will also be excluded)%N")
			s.append_string ("Output%N")
			s.append_string ("  -o --output {directory} : Output directory (current directory as default)%N")
			create Result.make (s)
		ensure
			usage_message_not_void: Result /= Void
		end

feature {NONE} -- Implementation

	seperate_string (a_string: STRING; seperator: CHARACTER): DS_ARRAYED_LIST [STRING] is
			-- Seperate 'a_string' at 'seperator'.
		local
			last_seperator: INTEGER
			index: INTEGER
		do
			create Result.make_default
			Result.set_equality_tester (create {KL_STRING_EQUALITY_TESTER})
			if a_string /= Void then
				from index := 1 until index > a_string.count loop
					if a_string.item (index) = seperator then
						Result.force_last (a_string.substring (last_seperator+1, index-1))
						last_seperator := index
					end
					index := index + 1
				end
				Result.force_last (a_string.substring (last_seperator+1, index-1))
			end
		ensure
			result_exists: Result /= Void
		end

feature -- Compile anchor

--	test_a: TEST_A
--	test_b: TEST_B

end