indexing

	description: "[

		Processor to build an HTML representation of an Eiffel class AST.
		If the class has compiled ancestors, the output will be in flat form.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EDOC_AST_HTML_PRINTER

inherit

	ET_AST_PRINTER
		undefine
			error_handler
		redefine
			file,
			process_break,
			process_class,
			process_class_type,
			process_creator,
			process_current,
			process_false_constant,
			process_feature_clause,
			process_features,
			process_identifier,
			process_indexing,
			process_indexing_list,
			process_invariants,
			process_keyword,
			process_keyword_manifest_string,
			process_parent_list,
			process_postconditions,
			process_preconditions,
			process_regular_manifest_string,
			process_result,
			process_symbol,
			process_special_manifest_string,
			process_tagged_assertion,
			process_true_constant,
			process_verbatim_string,
			process_void
		end

	EDOC_SHARED_CONTEXT
		export {NONE} all end

	EDOC_SHARED_CSS_CONSTANTS
		export {NONE} all end

	KL_IMPORTED_STRING_ROUTINES
		export {NONE} all end

create

	make_null

feature -- Access

	output_file: EDOC_HTML_OUTPUT_FILE
			-- Output file

	file: EDOC_HTML_OUTPUT_FILE
			-- Output file

	current_class: ET_CLASS
			-- Current class

	is_ignoring_identifiers_and_symbols: BOOLEAN
			-- Is 'Current' ignoring all identifiers and symbols?

	is_manifest_string_comment: BOOLEAN
			-- Are all manifest strings comments?

	feature_list: DS_LINKED_LIST [ET_FEATURE]
			-- List of all features of `Current'

	feature_clause_list: DS_LINKED_LIST [TUPLE [STRING, INTEGER]]
			-- List of id's of feature clauses

	feature_per_clause: DS_HASH_TABLE [DS_LINKED_LIST [ET_FEATURE], STRING]
			-- Features per clause

	create_features: DS_LINKED_LIST [ET_FEATURE]
			-- List of creation features

feature -- Element change

	set_output_file (an_output_file: EDOC_HTML_OUTPUT_FILE) is
			-- Set `output_file' to `an_output_file'.
		require
			an_output_file_not_void: an_output_file /= Void
		do
			output_file := an_output_file
			file := an_output_file
		ensure
			output_file_set: file = an_output_file
		end

	set_current_class (a_class: ET_CLASS) is
			-- Set `current_class' to `a_class'.
		do
			current_class := a_class
			build_feature_list
		end

feature -- Processing

	process_break (a_break: ET_BREAK) is
			-- Process `a_break'.
		do
			process_single_break (a_break)
		end

	process_class (a_class: ET_CLASS) is
			-- Process `a_class'.
		local
			line: STRING
		do
			file.start_tag_css ("div", css_class)

			if a_class.first_indexing /= Void then
				a_class.first_indexing.process (Current)
			end
			-- Class keywords
			create line.make_empty
			if a_class.frozen_keyword /= Void then
				line.append_string (a_class.frozen_keyword.text)
				line.append_character (' ')
			end
			if a_class.class_mark /= Void then
				line.append_string (a_class.class_mark.text)
				line.append_character (' ')
			end
			if a_class.external_keyword /= Void then
				line.append_string (a_class.external_keyword.text)
				line.append_character (' ')
			end
			line.append_string (a_class.class_keyword.text)
			file.tag_css_content ("h2", css_keyword, line)
			-- Class name
			file.start_tag_css ("div", css_indent)
			file.indent
			file.put_string (file.tag_css_content_to_string ("span", css_class_name, a_class.name.name))
			if a_class.formal_parameters /= Void then
				file.put_character (' ')
				a_class.formal_parameters.process (Current)
			end
			file.end_tag
			if a_class.obsolete_message /= Void then
				a_class.obsolete_message.process (Current)
			end
			if a_class.parents /= Void then
				a_class.parents.process (Current)
			end
			if a_class.creators /= Void then
				a_class.creators.process (Current)
			end

-- TODO
			if a_class.convert_features /= Void then
				file.start_tag ("pre")
				a_class.convert_features.process (Current)
				file.end_tag
			end

			process_features (a_class)

			if a_class.invariants /= Void then
				a_class.invariants.process (Current)
			else
				process_invariants (Void)
			end
			if a_class.second_indexing /= Void then
				a_class.second_indexing.process (Current)
			end

			file.tag_css_content ("h2", css_keyword, a_class.end_keyword.text)

			file.end_tag
		end

	process_class_type (a_type: ET_CLASS_TYPE) is
			-- Process `a_type'.
		local
			a_type_mark: ET_KEYWORD
			a_link: STRING
		do
			a_type_mark := a_type.type_mark
			if a_type_mark /= Void then
				a_type_mark.process (Current)
			end
			a_link := context.class_link_by_name (a_type.name)
			if a_link = Void then
				file.put_string (file.tag_css_content_to_string ("span", css_class_name, a_type.name.name))
			else
				file.put_string (file.link_css_content_to_string (a_link, css_linked_class_name, a_type.name.name))
			end
			process_break (a_type.name.break) -- TODO: remove
		end

	process_comment (a_text: STRING) is
			-- Process `a_text' as comment.
			-- Try to link feature names in the comment.
			-- TODO: refactor
		local
			splitter: ST_SPLITTER
			lines: DS_LIST [STRING]
			a_link, line: STRING
			substring, replacement: STRING
			i,j: INTEGER
			a_feature: ET_FEATURE
			in_text: BOOLEAN
		do
			a_text.prune_all ('%R')
			a_text.prune_all_leading (' ')
			a_text.prune_all_trailing (' ')
			a_text.prune_all_leading ('%N')
			a_text.prune_all_trailing ('%N')
			a_text.prune_all_leading ('%T')
			a_text.prune_all_trailing ('%T')

			from  i := 1; j := 1 until i = 0 or j = 0 loop
				i := a_text.index_of ('`', j)
				j := a_text.index_of ('%'', i+1)
				if i > 0 and j > 0 then
					substring := a_text.substring (i+1, j-1)

					a_feature := current_class.named_procedure (create {ET_IDENTIFIER}.make (substring))
					if a_feature = Void then
						a_feature := current_class.named_query (create {ET_IDENTIFIER}.make (substring))
					end
					if a_feature /= Void then
						a_link := feature_link_by_feature (a_feature)
						if a_link /= Void then
							replacement := file.link_css_content_to_string (a_link, css_linked_feature_name, substring)
						else
							replacement := file.tag_css_content_to_string ("span", css_commented_feature_name, substring)
						end
					else
						replacement := file.tag_css_content_to_string ("span", css_commented_feature_name, substring)
					end
					a_text.replace_substring (replacement, i, j)
					j := j + replacement.count - substring.count - 2
				end
			end

			create splitter.make
			splitter.set_separators ("%N")

			lines := splitter.split_greedy (a_text)
			file.start_tag_css ("div", css_comment)
			from
				in_text := False
				lines.start
			until
				lines.off
			loop
				line := lines.item_for_iteration
				line.prune_all_leading (' ')
				line.prune_all_trailing (' ')
				line.prune_all_leading ('%T')
				line.prune_all_trailing ('%T')
				if line.count > 0 then
					if not in_text and line.count > 1 then
						in_text := True
						file.start_tag ("p")
					end
					file.content_line (insert_class_links_in_comment (line)+"<br />")
				else
					if in_text then
						in_text := False
						file.end_tag
					end
				end
				lines.forth
			end
			if in_text then
				file.end_tag
			end
			file.end_tag
		end

	insert_class_links_in_comment (a_string: STRING): STRING is
			-- TODO: refactor
		local
			a_word, a_replacement: STRING
			a_feature_name: STRING
			i, j, k: INTEGER
			in_word: BOOLEAN
			a_class: ET_CLASS
			a_feature: ET_FEATURE
		do
			create Result.make_from_string (a_string)
			from
				i := 1
			until
				i > Result.count
			loop
				if in_word then
					if (Result.item (i).is_upper or Result.item (i) = '_') then

					else
						a_word := Result.substring (j, i-1)
						a_class := context.universe.class_by_name (a_word)
						if a_class /= Void then
							a_replacement := file.print_class_name_to_string (a_class)
							if j > 1 and then i < Result.count-1 and then Result.item (j-1) = '{' and then Result.item (i-1) = '}' and then Result.item (i) = '.' then
								from
									k := i + 1
								until
									k > Result.count or k = -1
								loop
									if Result.item (k).is_alpha or Result.item (k) = ' ' then
										k := k + 1
									else
										k := -1
									end
									if k > 0 then
										a_feature_name := Result.substring (i+1, k-1)
										a_feature := a_class.named_procedure (create {ET_IDENTIFIER}.make (a_feature_name))
										if a_feature = Void then
											a_feature := a_class.named_query (create {ET_IDENTIFIER}.make (a_feature_name))
										end
										-- TODO replace feature name
										if a_feature = Void then
											Result.replace_substring (a_replacement, j-1, i)
											i := i + a_replacement.count - a_word.count - 4
										else
											Result.replace_substring (a_replacement, j-1, i)
											i := i + a_replacement.count - a_word.count - 4
										end
									end
								end
							else
								Result.replace_substring (a_replacement, j, i-1)
								i := i + a_replacement.count - a_word.count - 2
							end
						end
						in_word := False
					end
				else
					if Result.item (i).is_upper or Result.item (i) = '_' then
						j := i
						in_word := True
					end
				end
				i := i + 1
			end
			if in_word then
				a_word := Result.substring (j, i-1)
				a_class := context.universe.class_by_name (a_word)
				if a_class /= Void then
					a_replacement := file.print_class_name_to_string (a_class)
					Result.replace_substring (a_replacement, j, i-1)
				end
			end
		end

	process_comment_2 (a_text: STRING) is
			-- Process 'a_comment'.
		require
			a_text_not_void: a_text /= Void
		local
			splitter: ST_SPLITTER
			lines: DS_LIST [STRING]
			line: STRING
		do
			-- Prepare text
			a_text.prune_all ('%R')
			a_text.prune_all ('%T')
			a_text.prune_all_leading (' ')
			a_text.prune_all_leading ('%N')
			a_text.prune_all_trailing (' ')
			a_text.prune_all_trailing ('%N')

			create splitter.make
			splitter.set_separators ("%N")
			lines := splitter.split (a_text)

			from
				lines.start
			until
				lines.after
			loop
				line := lines.item_for_iteration
				line.prune_all_leading (' ')
				line.prune_all_trailing (' ')
				line.prune_all_leading ('%T')
				line.prune_all_trailing ('%T')

				lines.forth
			end
		end

	process_creator (a_list: ET_CREATOR) is
			-- Process `a_list'.
		local
			a_clients: ET_CLIENTS
			i, nb: INTEGER
			a_feature: ET_FEATURE
		do
			a_clients := a_list.clients_clause
			if a_clients = Void then
				file.tag_css_content ("h2", css_keyword, a_list.creation_keyword.text)
			else
				file.start_tag ("h2")
				file.tag_css_content ("span", css_keyword, a_list.creation_keyword.text)
				a_clients.process (Current)
				file.end_tag
			end
			file.start_tag_css ("div", css_feature_clause_body)
			nb := a_list.count
			from i := 1 until i > nb loop
				a_feature := current_class.named_procedure (a_list.item (i).feature_name)
				if a_feature = Void then
					file.start_tag_css ("div", css_feature)
					file.tag_css_content ("div", css_feature_name, a_list.item (i).feature_name.feature_name.name)
					file.end_tag
				else
					process_feature (a_feature)
				end
				i := i + 1
			end
			file.end_tag
		end

	process_current (an_expression: ET_CURRENT) is
			-- Process `an_expression'.
		do
			process_special_keyword (an_expression)
		end

	process_false_constant (a_constant: ET_FALSE_CONSTANT) is
			-- Process `a_constant'.
		do
			process_special_keyword (a_constant)
		end

	process_feature_clause (a_feature_clause: ET_FEATURE_CLAUSE) is
			-- Process `a_feature_clause'.
		local
			a_clients: ET_CLIENTS
			a_break: ET_BREAK
			a_text: STRING
		do
			file.start_tag ("h2")
			file.tag_attributes_content ("a", << ["id", "clause-"+context.xhtml_anchor (feature_clause_name (a_feature_clause))], ["class", css_keyword] >>, a_feature_clause.feature_keyword.text)
			a_clients := a_feature_clause.clients_clause
			if a_clients /= Void then
				a_clients.process (Current)
			end
			a_break := a_feature_clause.break
			if a_break /= Void and then a_break.text.has_substring ("--") then
				create a_text.make_from_string (a_break.text)
				a_text.prune_all ('%T')
				a_text.prune_all ('%R')
				-- TODO: take only first line (if another comment is between feature clause and first feature)
				a_text.prune_all ('%N')
				a_text.prune_all_leading (' ')
				a_text.prune_all_trailing (' ')
				file.tag_css_content ("span", css_comment, " "+a_text)
			end
			file.end_tag
		end

	process_feature (a_feature: ET_FEATURE) is
			-- Process `a_feature'.
			-- TODO: refactor
		local
			a_query: ET_QUERY
			a_routine: ET_ROUTINE
			comment_text, a_link: STRING
			precursors: DS_BILINKED_LIST [ET_FEATURE]
			a_precursor: ET_FEATURE
			cursor: DS_LIST_CURSOR [ET_FEATURE]
			a_feature_name: STRING
		do
			a_query ?= a_feature
			a_routine ?= a_feature

			a_feature_name := context.escaped_feature_name (a_feature)

			from
				create precursors.make_default
				a_precursor := a_feature.first_precursor
			until
				a_precursor = Void
			loop
				precursors.put_first (a_precursor)
				a_precursor := a_precursor.first_precursor
			end

			file.start_tag_css ("div", css_feature)

			file.start_tag ("div")
			file.indent
			if a_feature.frozen_keyword /= Void then
				a_feature.frozen_keyword.process (Current)
				file.put_character (' ')
			end
			file.put_string (file.tag_attributes_content_to_string ("a", << ["id", context.xhtml_anchor (a_feature.extended_name.feature_name.name)] >>, file.tag_css_content_to_string ("span", css_feature_name, a_feature_name)))
			if a_feature.arguments /= Void then
				file.put_character (' ')
				a_feature.arguments.process (Current)
			end
			if a_query /= Void then
				a_query.declared_type.process (Current)
			end
			file.put_new_line
			file.end_tag
			file.start_tag_css ("div", css_feature_body)

			create comment_text.make_empty
			-- TODO: get comment text after generic attribute ("an_attribute: G -- comment")
			if a_feature.is_attribute or a_feature.is_constant_attribute or a_feature.is_unique_attribute then
				if a_feature.semicolon /= Void and then a_feature.semicolon.break /= Void then
					create comment_text.make_from_string (a_feature.semicolon.break.text)
				elseif a_feature.break /= Void then
					create comment_text.make_from_string (a_feature.break.text)
				end
			elseif a_routine /= Void then
				if a_routine.is_keyword /= Void and then a_routine.is_keyword.break /= Void then
					create comment_text.make_from_string (a_routine.is_keyword.break.text)
				elseif a_feature.break /= Void then
					create comment_text.make_from_string (a_feature.break.text)
				end
			elseif a_feature.break /= Void then
				create comment_text.make_from_string (a_feature.break.text)
			end

			if not precursors.is_empty then
				comment_text.append_string ("%N -- (From "+precursors.first.implementation_class.name.name+")%N")
--				a_link := context.feature_link_by_feature (precursors.first, precursors.first.implementation_class)
--				if a_link = Void then
--					comment_text.append_string ("%N -- (From "+file.tag_css_content_to_string ("span", css_comment_class_name, precursors.first.implementation_class.name.name)+")%N")
--				else
--					comment_text.append_string ("%N -- (From "+file.link_css_content_to_string (a_link, css_linked_class_name, precursors.first.implementation_class.name.name)+")%N")
--				end
			elseif a_feature.implementation_class /= current_class then
				comment_text.append_string ("%N -- (From "+a_feature.implementation_class.name.name+")%N")
--				a_link := context.feature_link_by_feature (a_feature, a_feature.implementation_class)
--				if a_link = Void then
--					comment_text.append_string ("%N -- (From "+file.tag_css_content_to_string ("span", css_comment_class_name, a_feature.implementation_class.name.name)+")%N")
--				else
--					comment_text.append_string ("%N -- (From "+file.link_css_content_to_string (a_link, css_linked_class_name, a_feature.implementation_class.name.name)+")%N")
--				end
			end
			file.start_tag_css ("div", css_feature_comment_body)
			process_comment (comment_text)
			file.end_tag

			if a_feature.obsolete_message /= Void then
				a_feature.obsolete_message.process (Current)
			end
			cursor := precursors.new_cursor
			from
				cursor.start
			until
				cursor.after
			loop
				if cursor.item.preconditions /= Void then
					cursor.item.preconditions.process (Current)
				end
				cursor.forth
			end
			if a_feature.preconditions /= Void then
				a_feature.preconditions.process (Current)
			end
			from
				cursor.start
			until
				cursor.after
			loop
				if cursor.item.postconditions /= Void then
					cursor.item.postconditions.process (Current)
				end
				cursor.forth
			end
			if a_feature.postconditions /= Void then
				a_feature.postconditions.process (Current)
			end
			file.end_tag
			file.end_tag
		end

	process_features (a_class: ET_CLASS) is
			-- Process feature clauses of `a_class'.
		local
			clause_cursor: DS_LIST_CURSOR [TUPLE [STRING, INTEGER]]
			feature_cursor: DS_LIST_CURSOR [ET_FEATURE]
			cluster_name: STRING
		do
			from
				clause_cursor := feature_clause_list.new_cursor
				clause_cursor.start
			until
				clause_cursor.after
			loop
				cluster_name ?= clause_cursor.item.item (1)
				feature_cursor := feature_per_clause.item (cluster_name).new_cursor
				from
					feature_cursor.start
					feature_cursor.item.feature_clause.process (Current)
					file.start_tag_css ("div", css_feature_clause_body)
				until
					feature_cursor.after
				loop
					process_feature (feature_cursor.item)
					feature_cursor.forth
				end
				file.end_tag
				clause_cursor.forth
			end
		end

	process_identifier (an_identifier: ET_IDENTIFIER) is
			-- Process `an_identifier'.
		local
			a_feature: ET_FEATURE
			a_link: STRING
		do
			if not is_ignoring_identifiers_and_symbols then
				a_feature := current_class.named_procedure (an_identifier)
				if a_feature = Void then
					a_feature := current_class.named_query (an_identifier)
				end
				if a_feature = Void then
					process_token (an_identifier)
				else
					a_link := feature_link_by_feature (a_feature)
					if a_link = Void then
						file.put_string (file.tag_css_content_to_string ("span", css_feature_name, an_identifier.name))
					else
						file.put_string (file.link_css_content_to_string (a_link, css_linked_feature_name, an_identifier.name))
					end
					process_break (an_identifier.break)
				end
			end
		end

	process_indexing (an_indexing: ET_INDEXING) is
			-- Process `an_indexing'.
		do
			if an_indexing.tag /= Void and then an_indexing.tag.identifier /= Void then
				file.tag_css_content ("h3", css_indexing_tag, an_indexing.tag.identifier.name)
				file.start_tag_css ("div", css_indexing_body)
				an_indexing.terms.process (Current)
				file.end_tag
			end
		end

	process_indexing_list (a_list: ET_INDEXING_LIST) is
			-- Process `a_list'.
		local
			i, nb: INTEGER
		do
			is_ignoring_identifiers_and_symbols := True
			is_manifest_string_comment := True
			file.start_tag_css ("div", css_indexing)
			file.tag_css_content ("h2", css_keyword, a_list.indexing_keyword.text)
			file.start_tag_css ("div", css_indexing_list)
			nb := a_list.count
			from i := 1 until i > nb loop
				a_list.item (i).process (Current)
				i := i + 1
			end
			file.end_tag
			file.end_tag
			is_ignoring_identifiers_and_symbols := False
			is_manifest_string_comment := True
		end

	process_invariants (a_list: ET_INVARIANTS) is
			-- Process `a_list'.
		local
			i, nb, j, ancestor_count: INTEGER
			ancestor: ET_CLASS
		do
			file.tag_attributes_content ("h2", << ["class", css_keyword], ["id", "invariant"] >>, "invariant")
			file.start_tag_css ("div", css_assertion_body)
			if a_list /= Void then
				nb := a_list.count
				from i := 1 until i > nb loop
					a_list.item (i).process (Current)
					i := i + 1
				end
			end
			ancestor_count := current_class.ancestors.count
			from j := 1 until j > ancestor_count loop
				ancestor := universe.class_by_name (current_class.ancestors.item (j).name.name)
				if ancestor /= Void and then ancestor.invariants /= Void then
					if j > 1 or a_list /= Void then
						file.tag ("br")
					end
					file.tag_css_content ("div", css_invariant_comment, "-- From "+file.print_class_name_to_string (ancestor))
					nb := ancestor.invariants.count
					from i := 1 until i > nb loop
						ancestor.invariants.item (i).process (Current)
						i := i + 1
					end
				end
				j := j + 1
			end
			file.end_tag
		end

	process_keyword (a_keyword: ET_KEYWORD) is
			-- Process `a_keyword'.
		do
			file.put_string (file.tag_css_content_to_string ("span", css_keyword, a_keyword.text))
			process_single_break (a_keyword.break)
		end

	process_keyword_manifest_string (a_string: ET_KEYWORD_MANIFEST_STRING) is
			-- Process `a_string'.
		do
			file.tag_css_content ("h2", css_keyword, a_string.keyword.text)
			a_string.manifest_string.process (Current)
		end

	process_parent_list (a_list: ET_PARENT_LIST) is
			-- Process `a_list'.
		do
			if not available_ancestors (current_class).is_empty then
				file.tag_css_content ("h2", css_keyword, a_list.inherit_keyword.text)
				print_ancestors_tree (current_class)
			end
		end

	process_postconditions (a_list: ET_POSTCONDITIONS) is
			-- Process `a_list'.
		local
			i, nb: INTEGER
			a_then_keyword: ET_TOKEN
			a_line: STRING
		do
			nb := a_list.count
			if nb > 0 then
				create a_line.make_empty
				a_line.append_string (file.tag_css_content_to_string ("span", css_keyword, a_list.ensure_keyword.text))
				a_then_keyword := a_list.then_keyword
				if a_then_keyword /= Void then
					a_line.append_character (' ')
					a_line.append_string (file.tag_css_content_to_string ("span", css_keyword, a_then_keyword.text))
				end
				file.tag_content ("div", a_line)
				file.start_tag_css ("div", css_assertion_body)
				from i := 1 until i > nb loop
					a_list.item (i).process (Current)
					i := i + 1
				end
				file.end_tag
			end
		end

	process_preconditions (a_list: ET_PRECONDITIONS) is
			-- Process `a_list'.
		local
			i, nb: INTEGER
			an_else_keyword: ET_TOKEN
			a_line: STRING
		do
			nb := a_list.count
			if nb > 0 then
				create a_line.make_empty
				a_line.append_string (file.tag_css_content_to_string ("span", css_keyword, a_list.require_keyword.text))
				an_else_keyword := a_list.else_keyword
				if an_else_keyword /= Void then
					a_line.append_character (' ')
					a_line.append_string (file.tag_css_content_to_string ("span", css_keyword, an_else_keyword.text))
				end
				file.tag_content ("div", a_line)
				file.start_tag_css ("div", css_assertion_body)
				nb := a_list.count
				from i := 1 until i > nb loop
					a_list.item (i).process (Current)
					i := i + 1
				end
				file.end_tag
			end
		end

	process_regular_manifest_string (a_string: ET_REGULAR_MANIFEST_STRING) is
			-- Process `a_string'.
		do
			if is_manifest_string_comment then
				process_comment (a_string.literal)
			else
				Precursor {ET_AST_PRINTER} (a_string)
			end
		end

	process_result (an_expression: ET_RESULT) is
			-- Process `an_expression'.
		do
			process_special_keyword (an_expression)
		end

	process_single_break (a_break: ET_BREAK) is
			-- Process `a_break' if it is a single blank.
		do
			if a_break /= Void and then a_break.text.is_equal (" ") then
				file.put_character (' ')
			end
		end

	process_special_keyword (a_keyword: ET_KEYWORD) is
			-- Process `an_expression'.
		do
			file.put_string (file.tag_css_content_to_string ("span", css_special_keyword, a_keyword.text))
			process_single_break (a_keyword.break)
		end

	process_special_manifest_string (a_string: ET_SPECIAL_MANIFEST_STRING) is
			-- Process `a_string'.
		do
			if is_manifest_string_comment then
				process_comment (a_string.literal)
			else
				Precursor {ET_AST_PRINTER} (a_string)
			end
		end

	process_symbol (a_symbol: ET_SYMBOL) is
			-- Process `a_symbol'.
		local
			text: STRING
		do
			if not is_ignoring_identifiers_and_symbols then
				text := STRING_.replaced_all_substrings (a_symbol.text, "<", "&lt;")
				text := STRING_.replaced_all_substrings (text, ">", "&gt;")
				file.put_string (file.tag_css_content_to_string ("span", css_symbol, text))
				process_single_break (a_symbol.break)
			end
		end

	process_tagged_assertion (an_assertion: ET_TAGGED_ASSERTION) is
			-- Process `an_assertion'.
		local
			an_expression: ET_EXPRESSION
		do
			file.indent
			file.put_string (file.tag_css_content_to_string ("span", css_assertion_tag, an_assertion.tag.identifier.name+":"))
			file.put_character (' ')
			an_expression := an_assertion.expression
			if an_expression /= Void then
				an_expression.process (Current)
			end
			file.put_string ("<br />%N")
		end

	process_true_constant (a_constant: ET_TRUE_CONSTANT) is
			-- Process `a_constant'.
		do
			process_special_keyword (a_constant)
		end

	process_verbatim_string (a_string: ET_VERBATIM_STRING) is
			-- Process `a_string'.
		do
			process_comment (a_string.literal)
		end

	process_void (an_expression: ET_VOID) is
			-- Process `an_expression'.
		do
			process_special_keyword (an_expression)
		end

feature -- Support

	feature_link_by_feature (a_feature: ET_FEATURE): STRING is
			-- Link to `a_feature' inside the same page
		do
			if feature_list.has (a_feature) then
				Result := "#"+context.xhtml_anchor (a_feature.extended_name.feature_name.name)
			end
		end

feature {NONE} -- Implementation

	build_feature_list is
			-- Build feature list of `current_class'
		local
			i, count: INTEGER
			a_feature: ET_FEATURE
			a_creator: ET_CREATOR
			j, ni, nj: INTEGER
		do
			create feature_list.make
			create feature_clause_list.make
			create feature_per_clause.make_default
			create create_features.make
			count := current_class.procedures.count
			from i := 1 until i > count loop
				a_feature := current_class.procedures.item (i)
				if not context.is_any_ignored or else not a_feature.implementation_class.name.name.is_equal ("ANY") then
					add_feature (a_feature)
				end
				i := i + 1
			end
			count := current_class.queries.count
			from i := 1 until i > count loop
				a_feature := current_class.queries.item (i)
				if not context.is_any_ignored or else not a_feature.implementation_class.name.name.is_equal ("ANY") then
					add_feature (a_feature)
				end
				i := i + 1
			end

			-- Index creators
			if current_class.creators /= Void then
				ni := current_class.creators.count
				from i := 1 until i > ni loop
					a_creator := current_class.creators.item (i)
					nj := a_creator.count
					from j := 1 until j > nj loop
						a_feature := current_class.named_procedure (a_creator.item (j).feature_name)
						if a_feature /= Void then
							create_features.force_last (a_feature)
						end
						j := j + 1
					end
					i := i + 1
				end
			end
		end

	add_feature (a_feature: ET_FEATURE) is
			-- Add a feature to the list.
		local
			id: STRING
			list: DS_LINKED_LIST [ET_FEATURE]
			cursor: DS_LINKED_LIST_CURSOR [ET_FEATURE]
			clause_index: INTEGER
			clause_cursor: DS_LINKED_LIST_CURSOR [TUPLE [STRING, INTEGER]]
		do
			-- Remove NONE features
			if not a_feature.clients.has_class (universe.none_class) then
				feature_list.force_last (a_feature)
				id := feature_clause_id (a_feature.feature_clause)
				if feature_per_clause.has (id) then
					list := feature_per_clause.item (id)
				else
					create list.make
					feature_per_clause.force (list, id)
					if context.feature_clause_order.has (feature_clause_name (a_feature.feature_clause)) then
						context.feature_clause_order.start
						context.feature_clause_order.search_forth (feature_clause_name (a_feature.feature_clause))
						clause_index := context.feature_clause_order.index
					else
						clause_index := context.user_defined_clause_index
					end
					from
						clause_cursor := feature_clause_list.new_cursor
						clause_cursor.start
					until
						clause_cursor.off or clause_cursor.item.integer_item (2) > clause_index
					loop
						clause_cursor.forth
					end
					if clause_cursor.before then
						feature_clause_list.force_first ([id, clause_index])
					else
						clause_cursor.put_left ([id, clause_index])
					end
				end
				from
					cursor := list.new_cursor
					cursor.start
				until
					cursor.off or cursor.item.name.name > a_feature.name.name
				loop
					cursor.forth
				end
				if cursor.before then
					list.force_first (a_feature)
				else
					cursor.put_left (a_feature)
				end
			end
			-- generate global index
			if a_feature.implementation_class = current_class then
				if context.global_index.is_full then
					context.global_index.resize (context.global_index.capacity*2)
				end
				context.global_index.force_last ([a_feature.name.name.as_lower, a_feature])
			end
		end

	feature_clause_name (a_feature_clause: ET_FEATURE_CLAUSE): STRING is
			-- Name for `a_feature_clause'.
		local
			comment: STRING
			i, j: INTEGER
		do
			create Result.make_empty
			comment := a_feature_clause.break.text
			i := comment.substring_index ("--", 1)
			if i > 0 then
				j := comment.index_of ('%N', i)
				if j = 0 then
					j := comment.index_of ('%R', i)
				end
				if j > 0 then
					Result.append_string (comment.substring (i+2, j-1))
				end
			end
			Result.prune_all ('%T')
			Result.prune_all_leading (' ')
			Result.prune_all_trailing (' ')
		end

	feature_clause_id (a_feature_clause: ET_FEATURE_CLAUSE): STRING is
			-- Generate id for `a_feature_clause'.
		do
			Result := feature_clause_name (a_feature_clause)
			if a_feature_clause.clients.has_class (universe.none_class) then
				-- TODO: add all clients
				Result.append (" {NONE}")
			end
		end

	available_ancestors (a_class: ET_CLASS): DS_LINKED_LIST [ET_CLASS] is
			-- Ancestors of `a_class' which aren't exported to none.
		local
			i: INTEGER
			parent: ET_PARENT
			exports: ET_EXPORT_LIST
			all_export: ET_ALL_EXPORT
			parent_class: ET_CLASS
		do
			-- TODO: take generic classes into account

			create Result.make
			if a_class.parents /= Void then
				from i := 1 until i > a_class.parents.count loop
					parent := a_class.parents.item (i).parent
					parent_class := universe.class_by_name (parent.type.name.name)
					if not context.is_any_ignored or else not parent_class.name.name.is_equal ("ANY") then
						exports := parent.exports
						if exports = Void then
							Result.force_last (parent_class)
						else
							all_export ?= exports.first
							if all_export = Void or else not all_export.clients_clause.has_class (universe.none_class) then
								Result.force_last (parent_class)
							end
						end
					end
					i := i + 1
				end
			end
		end

	print_ancestors_tree (a_class: ET_CLASS) is
			-- Print tree of ancestors for `a_class'.
		local
			ancestors: DS_LIST [ET_CLASS]
			cursor: DS_LIST_CURSOR [ET_CLASS]
			a_link: STRING
		do
			ancestors := available_ancestors (a_class)
			if not ancestors.is_empty then
				file.start_tag_css ("div", css_inherit_list)
				from
					cursor := ancestors.new_cursor
					cursor.start
				until
					cursor.after
				loop
					a_link := context.class_link_by_class (cursor.item)
					if a_link /= Void then
						file.tag_content ("div", file.link_css_content_to_string (a_link, css_linked_class_name, cursor.item.name.name))
					else
						file.tag_css_content ("div", css_class_name, cursor.item.name.name)
					end
					print_ancestors_tree (cursor.item)
					cursor.forth
				end
				file.end_tag
			end
		end

end