indexing

	description: "[

		Generate an HTML file of an Eiffel class.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EDOC_CLASS_HTML_FILE

inherit

	EDOC_HTML_OUTPUT_FILE
		export {NONE} all end

create

	make_from_class

feature {NONE} -- Initialisation

	make_from_class (a_class: ET_CLASS) is
			-- Initialise page with `a_class'.
		require
			a_class_not_void: a_class /= Void
		do
			make (context.class_file_name (a_class))
			et_class := a_class
			create html_printer.make_null (context.universe)
			html_printer.set_current_class (et_class)
		ensure
			class_set: et_class = a_class
		end

feature -- Access

	et_class: et_class
			-- Class

	html_printer: EDOC_AST_HTML_PRINTER
			-- HTML Printer

feature -- Basic operations

	generate_class_file is
			-- Generate class file for `et_class'.
		do
			open_write

			put_string ("<?xml version=%"1.0%" encoding=%"ISO-8859-1%"?>%N")
			put_string ("<!DOCTYPE html PUBLIC %"-//W3C//DTD XHTML 1.1//EN%" %"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd%">%N")
			start_tag_attributes ("html", << ["xmlns", "http://www.w3.org/1999/xhtml"] >>)
			start_tag ("head")
				tag_content ("title", et_class.name.name+" ("+context.short_documentation_title+")")
				tag_attributes ("link", << ["rel", "stylesheet"], ["type", "text/css"], ["href", context.relative_output_directory+"style.css"] >>)
			end_tag
			start_tag ("body")

				print_header_start
				print_class_file_buttons (et_class)
				print_header_end

				tag_css_content ("div", css_cluster_name_small, print_qualified_cluster_name_to_string (et_class.cluster, True))
				tag_content ("h1", "Class "+et_class.name.name)
				tag ("hr")

				print_ancestors
				print_descendants

				tag ("hr")

				print_feature_list
				print_class_body

				print_footer_start
				print_class_file_buttons (et_class)
				print_footer_end

			end_tag
			end_tag

			close
		end

feature {NONE} -- Implementation

	print_ancestors is
			-- Print list of ancestors of `et_class'.
		local
			class_name_list: DS_LIST [ET_CLASS_NAME]
		do
			class_name_list := ancestors_list (et_class)
			if class_name_list.count > 0 then
				tag_content ("h2", "Direct ancestors")
				print_class_name_list (css_ancestors_list, class_name_list)
			end
		end

	print_descendants is
			-- Print list of descendants of `et_class'.
		local
			class_name_list: DS_LIST [ET_CLASS_NAME]
		do
			class_name_list := descendants_list (et_class)
			if class_name_list.count > 0 then
				tag_content ("h2", "Known direct descendants")
				print_class_name_list (css_descendants_list, class_name_list)
			end
		end

	print_class_name_list (a_css_class: STRING; class_name_list: DS_LIST [ET_CLASS_NAME]) is
			-- Print a list with all class names of `class_name_list'.
		local
			cursor: DS_LIST_CURSOR [ET_CLASS_NAME]
		do
			if class_name_list.count > 0 then
				start_tag_css ("div", a_css_class)
				indent
				from
					cursor := class_name_list.new_cursor
					cursor.start
				until
					cursor.after
				loop
					print_class_name (context.universe.class_by_name (cursor.item.name))
					cursor.forth
					if not cursor.after then
						put_string (", ")
					end
				end
				put_string ("%N")
				end_tag
			end
		end

	print_class_body is
			-- Print body of class page.
		do
			html_printer.set_output_file (Current)
			html_printer.process_class (et_class)
		end

	print_feature_list is
			-- Print a list of all features of `et_class'.
		local
			feature_clause_id_cursor: DS_LIST_CURSOR [TUPLE [STRING, INTEGER]]
			feature_cursor: DS_LIST_CURSOR [ET_FEATURE]
			cluster_id, a_link: STRING
		do
			start_tag_css ("div", css_feature_list)
			if not html_printer.create_features.is_empty then
				tag_attributes_content ("h2", << ["id", "features"] >>, "Creation")
				start_tag_css ("ul", css_feature_list_feature)
				from
					feature_cursor := html_printer.create_features.new_cursor
					feature_cursor.start
				until
					feature_cursor.after
				loop
					a_link := "#"+feature_cursor.item.name.name
					tag_content ("li", link_css_content_to_string (a_link, css_linked_feature_name, context.escaped_feature_name (feature_cursor.item)))
					feature_cursor.forth
				end
				end_tag

				tag_content ("h2", "Features")
			else
				tag_attributes_content ("h2", << ["id", "features"] >>, "Features")
			end
			start_tag_css ("ul", css_feature_list_clause)
			feature_clause_id_cursor := html_printer.feature_clause_list.new_cursor
			from
				feature_clause_id_cursor.start
			until
				feature_clause_id_cursor.after
			loop
				cluster_id ?= feature_clause_id_cursor.item.item (1)
				feature_cursor := html_printer.feature_per_clause.item (cluster_id).new_cursor
				from
					feature_cursor.start
					start_tag ("li")
					tag_content ("strong",link_content_to_string ("#"+"clause-"+context.xhtml_anchor (feature_clause_name (feature_cursor.item.feature_clause)), feature_clause_name (feature_cursor.item.feature_clause)))
					start_tag_css ("ul", css_feature_list_feature)
				until
					feature_cursor.after
				loop
					a_link := html_printer.feature_link_by_feature (feature_cursor.item)
					if a_link = Void then
						tag_css_content ("li", css_feature_name, feature_cursor.item.name.name)
					else
						tag_content ("li", link_css_content_to_string (a_link, css_linked_feature_name, context.escaped_feature_name (feature_cursor.item)))
					end
					feature_cursor.forth
				end
				end_tag
				end_tag
				feature_clause_id_cursor.forth
			end
			end_tag
			tag_content ("h2", link_content_to_string ("#invariant", "Invariants"))
			end_tag
		end

	feature_clause_name (a_feature_clause: ET_FEATURE_CLAUSE): STRING is
			-- Name for `a_feature_clause'.
			-- TODO: refactor
		local
			comment: STRING
			i, j: INTEGER
		do
			create Result.make_empty
			comment := a_feature_clause.break.text
			i := comment.substring_index ("--", 1)
			if i > 0 then
				j := comment.index_of ('%N', i)
				if j = 0 then
					j := comment.index_of ('%R', i)
				end
				if j > 0 then
					Result.append_string (comment.substring (i+2, j-1))
				end
			end
			Result.prune_all ('%T')
			Result.prune_all_leading (' ')
			Result.prune_all_trailing (' ')
		end

	ancestors_list (a_class: ET_CLASS): DS_LINKED_LIST [ET_CLASS_NAME] is
			-- List of ancestors of 'a_class'
		local
			i: INTEGER
			class_name: ET_CLASS_NAME
		do
			create Result.make
			if a_class.parents /= Void then
				from i := 1 until i > a_class.parents.count loop
					class_name := a_class.parents.item (i).parent.type.name
					if not context.is_any_ignored or else not class_name.name.is_equal ("ANY") then
						Result.force_last (class_name)
					end
					i := i + 1
				end
			end
		end

	descendants_list (a_class: ET_CLASS): DS_LINKED_LIST [ET_CLASS_NAME] is
			-- List of direct descendants of 'a_class'
		local
			class_cursor: DS_HASH_TABLE_CURSOR [ET_CLASS, ET_CLASS_NAME]
			ancestors_cursor: DS_LIST_CURSOR [ET_CLASS_NAME]
			ancestor_found: BOOLEAN
		do
			create Result.make
			class_cursor := context.universe.classes.new_cursor
			from
				class_cursor.start
			until
				class_cursor.after
			loop
				ancestors_cursor := ancestors_list (class_cursor.item).new_cursor
				from
					ancestor_found := False
					ancestors_cursor.start
				until
					ancestors_cursor.after or ancestor_found
				loop
					if ancestors_cursor.item.same_class_name (a_class.name) then
						Result.force_last (class_cursor.key)
						ancestor_found := True
					end
					ancestors_cursor.forth
				end
				class_cursor.forth
			end
		end

end