indexing

	description: "[

		Generate an HTML file for an Eiffel cluster.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EDOC_CLUSTER_HTML_FILE

inherit

	EDOC_HTML_OUTPUT_FILE
		export {NONE} all end

create

	make_from_cluster

feature {NONE} -- Initialisation

	make_from_cluster (a_cluster: ET_CLUSTER) is
			-- Initialise with `a_cluster'.
		require
			a_cluster_not_void: a_cluster /= Void
		do
			make (context.cluster_file_name (a_cluster))
			cluster := a_cluster
		end

feature -- Access

	cluster: ET_CLUSTER
			-- Cluster

feature -- Basic operations

	generate_cluster_file is
			-- Generate cluster file for `cluster'.
		local
			class_list: DS_ARRAYED_LIST [ET_CLASS]
			class_cursor: DS_LIST_CURSOR [ET_CLASS]
		do
			open_write

			put_string ("<?xml version=%"1.0%" encoding=%"ISO-8859-1%"?>%N")
			put_string ("<!DOCTYPE html PUBLIC %"-//W3C//DTD XHTML 1.1//EN%" %"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd%">%N")
			start_tag_attributes ("html", << ["xmlns", "http://www.w3.org/1999/xhtml"] >>)
			start_tag ("head")
				tag_content ("title", cluster.name+" ("+context.short_documentation_title+")")
				tag_attributes ("link", << ["rel", "stylesheet"], ["type", "text/css"], ["href", context.relative_output_directory+"style.css"] >>)
			end_tag
			start_tag ("body")

				print_header_start
				print_cluster_file_buttons (cluster)
				print_header_end

				tag_content ("h1", "Cluster "+print_qualified_cluster_name_to_string (cluster, False))

				if cluster.subclusters /= Void then
					tag ("hr")
					tag_content ("h2", "Clusters")
					start_tag_css ("ul", css_cluster_tree)
					print_subclusters_tree (cluster)
					end_tag
				end

				class_list := context.universe.classes_by_cluster (cluster)
				if class_list.count > 0 then
					class_list.sort (create {DS_QUICK_SORTER [ET_CLASS]}.make (create {CLASS_COMPARATOR}))
					tag ("hr")
					tag_content ("h2", "Classes")
					start_tag ("ul")
					from
						class_cursor := class_list.new_cursor
						class_cursor.start
					until
						class_cursor.after
					loop
						start_tag ("li")
						print_class_name (class_cursor.item)
						end_tag
						class_cursor.forth
					end
					end_tag
				end

				print_footer_start
				print_cluster_file_buttons (cluster)
				print_footer_end

			end_tag
			end_tag

			close
		end

feature {NONE} -- Implementation

end