indexing

	description: "[

		Context of a generated documentation. This class generates the documentation and provides
		all links to clusters, classes and features.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EDOC_CONTEXT

inherit

	EDOC_SHARED_CONTEXT
		export {NONE} all end

	DT_SHARED_SYSTEM_CLOCK
		export {NONE} all end

	KL_SHARED_EXECUTION_ENVIRONMENT
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

	KL_IMPORTED_STRING_ROUTINES
		export {NONE} all end

create

	make

feature {NONE} -- Initialisation

	make is
			-- Initalise default values.
		do
			set_output_directory (create {KL_DIRECTORY}.make ("."))
			set_css_filename (default_stylesheet_file)

			create included_clusters.make_equal
			create ignored_clusters.make_equal
			create classes.make (500)
			create clusters.make (50)
			create global_index.make (6000)

			is_any_ignored := True
			is_inherit_export_none_ignored := True
			is_feature_clause_export_none_ignored := True

			create inherit_classes_ignored.make
			create feature_clauses_ignored.make

			documentation_title := "{title-long}"
			short_documentation_title := "{title-short}"
		end

feature -- Access

	output_directory: KI_DIRECTORY
			-- Output directory

	absolute_output_directory: STRING
			-- Absolute path of output directory

	css_filename: STRING
			-- CSS filename

	documentation_title: STRING
			-- Title of documentation

	short_documentation_title: STRING
			-- Short title of documentation

	included_clusters: DS_LINKED_LIST [STRING]
			-- Names of top level clusters to include

	ignored_clusters: DS_LINKED_LIST [STRING]
			-- Names of clusters to ignore

	universe: ET_UNIVERSE
			-- Eiffel parser universe

	classes: DS_ARRAYED_LIST [ET_CLASS]
			-- All classes to process

	clusters: DS_ARRAYED_LIST [ET_CLUSTER]
			-- All clusters to process

	inherit_classes_ignored: DS_LINKED_LIST [STRING]
			-- Classes which are ignored in inherit clause

	feature_clauses_ignored: DS_LINKED_LIST [STRING]
			-- Feature clauses which are ignored

	global_index: DS_ARRAYED_LIST [TUPLE [STRING, ANY]]
			-- Global index over all clusters, classes and features

feature -- Current file options

	previous_link: STRING
			-- Link to previous page

	next_link: STRING
			-- Link to next page

	up_link: STRING
			-- Link up

feature -- Status report

	is_any_ignored: BOOLEAN
			-- Is class ANY ingored when documentation is created?

	is_inherit_export_none_ignored: BOOLEAN
			-- Is any inherited class which is exported to NONE ignored?

	is_feature_clause_export_none_ignored: BOOLEAN
			-- Is any feature clause which is exported to NONE ignored?

	is_including_cluster (a_cluster: ET_CLUSTER): BOOLEAN is
			-- Is `a_cluster' included in documentation?
		do
			Result := clusters.has (a_cluster)
		end

	is_including_class (a_class: ET_CLASS): BOOLEAN is
			-- Is `a_class' included in documentation?
		do
			Result := classes.has (a_class)
		end

feature -- Element change

	set_universe (a_universe: ET_UNIVERSE) is
			-- Set `universe' to `a_universe'.
		do
			universe := a_universe
		ensure
			universe_set: universe = a_universe
		end

	set_output_directory (a_directory: KL_DIRECTORY) is
			-- Set `output_directory' to `a_directory'.
		do
			output_directory := a_directory
			absolute_output_directory := file_system.absolute_pathname (a_directory.name)
		end

	set_css_filename (a_filename: STRING) is
			-- Set `css_filename' to `a_filename'.
		do
			css_filename := a_filename
		end

	set_documentation_title (a_title: STRING) is
			-- Set `documentation_title' to `a_title'.
		require
			a_title_not_void: a_title /= Void
		do
			documentation_title := a_title
		ensure
			documentation_title_set: documentation_title = a_title
		end

	set_short_documentation_title (a_title: STRING) is
			-- Set `short_documentation_title' to `a_title'.
		require
			a_title_not_void: a_title /= Void
		do
			short_documentation_title := a_title
		ensure
			short_documentation_title_set: documentation_title = a_title
		end

feature -- File path / links

	relative_output_directory: STRING Is
			-- Relative path to output directory
		local
			i: INTEGER
		do
			create Result.make_empty
			from i := 0 until i = nested_directory_level loop
				Result.append_string ("../")
				i := i + 1
			end
		ensure
			result_exists: Result /= Void
		end

	cluster_file_name (a_cluster: ET_CLUSTER): STRING is
			-- Generate file name for `a_cluster'.
		require
			a_cluster_not_vod: a_cluster /= Void
		do
			Result := "cluster."+a_cluster.prefixed_name.as_lower+".html"
		ensure
			result_exists: Result /= Void
		end

	cluster_file_directory (a_cluster: ET_CLUSTER): STRING is
			-- Generate cluster file path for `a_cluster' relative to output directory.
		require
			a_cluster_not_void: a_cluster /= Void
		local
			a_parent: ET_CLUSTER
		do
			create Result.make_from_string (a_cluster.name)
			from
				a_parent := a_cluster.parent
			until
				a_parent = Void
			loop
				Result.prepend_string (a_parent.name+"/")
				a_parent := a_parent.parent
			end
		ensure
			result_exists: Result /= Void
		end

	relative_cluster_directory (a_cluster: ET_CLUSTER): STRING is
			-- Relative path to `a_cluster'.
		require
			a_cluster_not_void: a_cluster /= Void
		do
			Result := relative_output_directory+cluster_file_directory (a_cluster)
		ensure
			result_exists: Result /= Void
		end

	cluster_link_by_cluster (a_cluster: ET_CLUSTER): STRING is
			-- Generate link to `a_cluster'.
		require
			a_cluster_not_vod: a_cluster /= Void
		do
			if not ignored_clusters.has (a_cluster.name) then
				Result := relative_cluster_directory (a_cluster)+"/"+cluster_file_name (a_cluster)
			end
		end

	class_file_name (a_class: ET_CLASS): STRING is
			-- Generate file name for `a_class'.
		require
			a_class_not_void: a_class /= Void
		do
			Result := a_class.name.name.as_lower+".html"
		ensure
			result_exists: Result /= Void
		end

	class_link_by_class (a_class: ET_CLASS): STRING is
			-- Generate link to `a_class'.
		require
			a_class_not_void: a_class /= Void
		do
			if classes.has (a_class) then
				Result := relative_cluster_directory (a_class.cluster)+"/"+class_file_name (a_class)
			end
		ensure
			link_to_included_class_exists: is_including_class (a_class) implies Result /= Void
		end

	class_link_by_name (a_class_name: ET_CLASS_NAME): STRING is
			-- Generate link to `a_class_name'.
			-- Return `Void' if `a_class_name' is not part of `universe'.
		require
			a_class_name_not_void: a_class_name /= Void
		local
			a_class: ET_CLASS
		do
			a_class := universe.class_by_name (a_class_name.name)
			if a_class = Void or else a_class.cluster = Void then
				Result := Void
			else
				Result := class_link_by_class (a_class)
			end
		end

	feature_link_by_feature (a_feature: ET_FEATURE; a_class: ET_CLASS): STRING is
			-- Generate link to `a_feature' in `a_class'.
		require
			a_feature_not_void: a_feature /= Void
		local
			class_link: STRING
		do
			class_link := class_link_by_class (a_class)
			if class_link /= Void then
				Result := class_link+"#"+xhtml_anchor (a_feature.extended_name.feature_name.name)
			end
		end

	relative_overview_link: STRING is
			-- Link to overview page
		do
			Result := relative_output_directory+"index.html"
		end

	relative_index_link (a_letter: CHARACTER): STRING is
			-- Link to index page of 'a_letter'
		do
			Result := relative_output_directory+"index/index-"+a_letter.as_lower.out+".html"
		end

	relative_classes_link: STRING is
			-- Link to classes page
		do
			Result := relative_output_directory+"index/classes.html"
		end

	relative_clusters_link: STRING is
			-- Link to clusters page
		do
			Result := relative_output_directory+"index/clusters.html"
		end

	escaped_feature_name (a_feature: ET_FEATURE): STRING is
			-- XHTML conform feature name
		do
			Result := string_.replaced_all_substrings (a_feature.extended_name.feature_name.name.as_lower, " ", "&nbsp;")
			Result := string_.replaced_all_substrings (Result, "%"", "&quot;")
			Result := string_.replaced_all_substrings (Result, "<", "&lt;")
			Result := string_.replaced_all_substrings (Result, ">", "&gt;")
		end

	xhtml_anchor (a_string: STRING): STRING is
			-- XHTML conform anchor for 'a_string'
		do
			Result := string_.replaced_all_substrings (a_string, " ", "_")
			Result := string_.replaced_all_substrings (Result, "%"", "_quot_")
			Result := string_.replaced_all_substrings (Result, "<", "_lt_")
			Result := string_.replaced_all_substrings (Result, ">", "_gt_")
			Result := string_.replaced_all_substrings (Result, "@", "_at_")
			Result := string_.replaced_all_substrings (Result, "+", "_plus_")
			Result := string_.replaced_all_substrings (Result, "-", "_minus_")
			Result := string_.replaced_all_substrings (Result, "/", "_slash_")
			Result := string_.replaced_all_substrings (Result, "\", "_backslash_")
			Result := string_.replaced_all_substrings (Result, "=", "_equal_")
			Result := string_.replaced_all_substrings (Result, "*", "_asterix_")
		end

feature -- Basic operations

	generate_documentation is
			-- Generate documentation.
		require
			universe_set: universe /= Void
		local
			overview_file: EDOC_OVERVIEW_HTML_FILE
		do
			parse_universe
			index_clusters
			index_classes

			processing_count := classes.count + clusters.count
			nested_directory_level := 0

			process_clusters (universe.clusters.clusters)

			-- Generate index file
			nested_directory_level := nested_directory_level + 1
			file_system.recursive_create_directory (absolute_output_directory+"/index")
			Error_handler.report_info_message ("Entering directory: "+absolute_output_directory+"/index")
			file_system.set_current_working_directory (absolute_output_directory+"/index")
			Error_handler.report_info_message ("Generating index files")
			process_global_index
			nested_directory_level := nested_directory_level - 1

			Error_handler.report_info_message ("Entering directory: "+absolute_output_directory)
			file_system.set_current_working_directory (absolute_output_directory)
			-- Generate overview file
			Error_handler.report_info_message ("Generating overview")
			create overview_file.make_overview_file
			overview_file.generate
			-- Generate stylesheet
			Error_handler.report_info_message ("Generating stylesheet")
			file_system.copy_file (css_filename, file_system.pathname (absolute_output_directory, "style.css"))
			Error_handler.report_info_message ("Documentation finished")
		end

	process_clusters (a_cluster_list: DS_ARRAYED_LIST [ET_CLUSTER]) is
			-- Generate documentation for 'a_cluster_list'.
		require
			a_cluster_list_not_void: a_cluster_list /= Void
		local
			i, a_count: INTEGER
			a_cluster: ET_CLUSTER
			a_cluster_cursor: DS_LIST_CURSOR [ET_CLUSTER]
		do
			a_cluster_list.sort (cluster_sorter)
			a_cluster_cursor := a_cluster_list.new_cursor
			from
				a_cluster_cursor.start
			until
				a_cluster_cursor.after
			loop
				if is_including_cluster (a_cluster_cursor.item) then
					a_cluster_cursor.forth
				else
					a_cluster_cursor.remove
				end
			end
			a_count := a_cluster_list.count
			nested_directory_level := nested_directory_level + 1
			from i := 1 until i > a_count loop
				a_cluster := a_cluster_list.item (i)
				-- generate links
				if i > 1 then
					previous_link := cluster_link_by_cluster (a_cluster_list.item (i - 1))
				else
					previous_link := Void
				end
				if a_cluster.parent = Void then
					up_link := relative_output_directory+"index.html"
				else
					up_link := cluster_link_by_cluster (a_cluster.parent)
				end
				if i < a_count then
					next_link := cluster_link_by_cluster (a_cluster_list.item (i + 1))
				else
					next_link := Void
				end
				-- process cluster
				process_cluster (a_cluster)
				i := i + 1
			end
			nested_directory_level := nested_directory_level - 1
		end

	process_cluster (a_cluster: ET_CLUSTER) is
			-- Generate documentation for 'a_cluster'.
		require
			a_cluster_not_void: a_cluster /= Void
		local
			a_cluster_file_directory: STRING
			a_cluster_file: EDOC_CLUSTER_HTML_FILE
			a_directory: KL_DIRECTORY
		do
			Error_handler.report_info_message (processing_count.out+" Processing cluster '"+a_cluster.name+"'")
			processing_count := processing_count - 1
			-- generate cluster file
			a_cluster_file_directory := file_system.pathname (absolute_output_directory, cluster_file_directory (a_cluster))
			Error_handler.report_info_message ("Entering directory: "+a_cluster_file_directory)
			create a_directory.make (a_cluster_file_directory)
			a_directory.recursive_create_directory
			file_system.set_current_working_directory (a_directory.name)
			create a_cluster_file.make_from_cluster (a_cluster)
			a_cluster_file.generate_cluster_file
			-- process classes and subclusters
			process_classes (universe.classes_by_cluster (a_cluster))
			if a_cluster.subclusters /= Void then
				process_clusters (a_cluster.subclusters.clusters)
			end
		end

	process_classes (a_class_list: DS_ARRAYED_LIST [ET_CLASS]) is
			-- Generate documentation for 'a_class_list'.
		require
			a_class_list_not_void: a_class_list /= Void
		local
			a_class: ET_CLASS
			i, a_count: INTEGER
		do
			a_count := a_class_list.count
			a_class_list.sort (class_sorter)
			from i := 1 until i > a_count loop
				a_class := a_class_list.item (i)
				-- generate links
				if i > 1 then
					previous_link := class_link_by_class (a_class_list.item (i - 1))
				else
					previous_link := Void
				end
				up_link := cluster_link_by_cluster (a_class.cluster)
				if i < a_count then
					next_link := class_link_by_class (a_class_list.item (i + 1))
				else
					next_link := Void
				end
				-- process class
				process_class (a_class)
				i := i + 1
			end
		end

	process_class (a_class: ET_CLASS) is
			-- Generate documentation for 'a_class'
		require
			a_class_not_void: a_class /= Void
		local
			a_class_file: EDOC_CLASS_HTML_FILE
		do
			Error_handler.report_info_message (processing_count.out+" Processing class '"+a_class.name.name+"'")
			processing_count := processing_count - 1
			-- generate class file
			create a_class_file.make_from_class (a_class)
			a_class_file.generate_class_file
		end

	process_global_index is
			-- Generate index over clusters, classes and features.
		local
			an_index_file: EDOC_INDEX_HTML_FILE
			a_classes_file: EDOC_CLASSES_HTML_FILE
			a_clusters_file: EDOC_CLUSTERS_HTML_FILE
			a_letter: CHARACTER
		do
			global_index.sort (create {DS_QUICK_SORTER [TUPLE [STRING, ANY]]}.make (create {TUPLE_STRING_COMPARATOR}))
			up_link := relative_output_directory+"index.html"
			from
				a_letter := 'a'
			until
				a_letter > 'z'
			loop
				-- generate links
				if a_letter > 'a' then
					previous_link := "index-"+a_letter.previous.out+".html"
				else
					previous_link := Void
				end
				if a_letter < 'z' then
					next_link := "index-"+a_letter.next.out+".html"
				else
					next_link := Void
				end
				-- generate file
				create an_index_file.make_index_file (a_letter)
				an_index_file.generate_index_file
				a_letter := a_letter.next
			end
			-- generate classes and clustesr file
			create a_classes_file.make_file
			a_classes_file.generate_file
			create a_clusters_file.make_file
			a_clusters_file.generate_file
		end

	default_stylesheet_file: STRING is
			-- TODO: refactor to option / remove EM dependency
		local
			em_value: STRING
		once
			em_value := execution_environment.variable_value ("EM")
			if em_value /= Void then
				Result := file_system.nested_pathname (em_value, << "resource", "tools", "eiffeldoc" >>)
			else
				Result := (".").twin
			end
			Result := file_system.pathname (Result, "style.css")
		end

	feature_clause_order: DS_ARRAYED_LIST [STRING] is
			-- TODO: refactor ro option / remove EM dependency
		local
			em_value: STRING
			file_name: STRING
			file: KL_TEXT_INPUT_FILE
			line: STRING
		once
			create Result.make_equal (30)
			em_value := execution_environment.variable_value ("EM")
			if em_value /= Void then
				file_name := file_system.nested_pathname (em_value, << "resource", "tools", "eiffeldoc" >>)
			else
				file_name := (".").twin
			end
			file_name := file_system.pathname (file_name, "feature_clause_order.txt")
			create file.make (file_name)
			if file.exists then
				from
					file.open_read
				until
					file.end_of_file
				loop
					file.read_line
					create line.make_from_string (file.last_string)
					if line.count > 0 then
						Result.force_last (line)
					end
				end
			end
		end

	user_defined_clause_index: INTEGER is
			-- Index in features clause list of user defined entry
		once
			feature_clause_order.start
			feature_clause_order.search_forth ("$USER_DEFINED$")
			if feature_clause_order.off then
				Result := feature_clause_order.count + 1
			else
				Result := feature_clause_order.index
			end
		end

feature {NONE} -- Implementation

	processing_count: INTEGER
			-- Number of classes and clusters to process

	nested_directory_level: INTEGER
			-- Number of nested directories from output directory to current working directory

	parse_universe is
			-- Parse and compile universe.
		local
			start_time: INTEGER
		do
			start_time := system_clock.time_now.millisecond_count
			-- Parsing
			Error_handler.report_info_message ("EiffelParser: Parsing all - Start: "+system_clock.time_now.time_out)
			universe.parse_all
			universe.check_provider_validity
			Error_handler.report_info_message ("EiffelParser: Parsing done - Finish: "+system_clock.time_now.time_out)
			Error_handler.report_info_message ("EiffelParser: Compiling degree 4 - Start: "+system_clock.time_now.time_out)
			universe.compile_degree_4
			Error_handler.report_info_message ("EiffelParser: Compiling done - Finish: "+system_clock.time_now.time_out)
--			Error_handler.report_info_message ("EiffelParser: Compiling degree 3 - Start: "+system_clock.time_now.time_out)
--			universe.compile_degree_3 (True)
--			Error_handler.report_info_message ("EiffelParser: Compiling done - Finish: "+system_clock.time_now.time_out)
			Error_handler.report_info_message ("EiffelParser: Total time: "+(create {DT_TIME}.make_from_millisecond_count (system_clock.time_now.millisecond_count-start_time)).time_out)
		end

	index_clusters is
			-- Index all clusters which will be processed.
		local
			a_cluster_cursor: DS_LIST_CURSOR [ET_CLUSTER]
			a_cluster: ET_CLUSTER
		do
			Error_handler.report_info_message ("Indexing clusters")
			universe.clusters.clusters.sort (cluster_sorter)
			a_cluster_cursor := universe.clusters.clusters.new_cursor
			from
				a_cluster_cursor.start
			until
				a_cluster_cursor.after
			loop
				a_cluster := a_cluster_cursor.item
				if included_clusters.count = 0 or else included_clusters.has (a_cluster.prefixed_name) then
					clusters.force_last (a_cluster)
					global_index.force_last ([a_cluster.name.as_lower, a_cluster])
					if a_cluster.subclusters /= Void then
						index_subclusters (a_cluster.subclusters)
					end
				else
					error_handler.report_info_message ("Ignoring cluster '"+a_cluster.prefixed_name+"'")
				end
				a_cluster_cursor.forth
			end
			clusters.sort (cluster_sorter)
			Error_handler.report_info_message ("done ("+clusters.count.out+" clusters selected)")
		end

	index_subclusters (subclusters: ET_CLUSTERS) is
			-- Index subclusters according to 'ignored_clusters'.
		local
			a_cluster_cursor: DS_LIST_CURSOR [ET_CLUSTER]
			a_cluster: ET_CLUSTER
		do
			subclusters.clusters.sort (cluster_sorter)
			a_cluster_cursor := subclusters.clusters.new_cursor
			from
				a_cluster_cursor.start
			until
				a_cluster_cursor.after
			loop
				a_cluster := a_cluster_cursor.item
				if ignored_clusters.has (a_cluster.prefixed_name) then
					Error_handler.report_info_message ("Ignoring cluster '"+a_cluster.prefixed_name+"'")
				else
					clusters.force_last (a_cluster)
					global_index.force_last ([a_cluster.name.as_lower, a_cluster])
					if a_cluster.subclusters /= Void then
						index_subclusters (a_cluster.subclusters)
					end
				end
				a_cluster_cursor.forth
			end
		end

	index_classes is
			-- Index all classes which will be processed.
		local
			a_class_cursor: DS_HASH_TABLE_CURSOR [ET_CLASS, ET_CLASS_NAME]
			a_class: ET_CLASS
		do
			Error_handler.report_info_message ("Indexing classes")
			a_class_cursor := universe.classes.new_cursor
			from
				a_class_cursor.start
			until
				a_class_cursor.after
			loop
				a_class := a_class_cursor.item
				if a_class.cluster /= Void and then is_including_cluster (a_class.cluster) then
					classes.force_last (a_class)
					global_index.force_last ([a_class.name.name.as_lower, a_class])
				end
				a_class_cursor.forth
			end
			classes.sort (class_sorter)
			Error_handler.report_info_message ("done ("+classes.count.out+" classes selected)")
		end

end