indexing

	description: "[

		An HTML File, which can generate also print cluster and class names and other
		often needed output.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EDOC_HTML_OUTPUT_FILE

inherit

	KL_TEXT_OUTPUT_FILE
		redefine
			make
		end

	EDOC_SHARED_CSS_CONSTANTS
		export {NONE} all end

	EDOC_SHARED_CONTEXT
		export {NONE} all end

	KL_SHARED_FILE_SYSTEM
		export {NONE} all end

create

	make

feature {NONE} -- Initialisation

	make (a_name: like name) is
			-- Create a new file named `a_name'.
			-- (`a_name' should follow the pathname convention
			-- of the underlying platform. For pathname conversion
			-- use KI_FILE_SYSTEM.pathname_from_file_system.)
		do
			Precursor {KL_TEXT_OUTPUT_FILE} (a_name)
			if not file_system.directory_exists (file_system.dirname (a_name)) then
				file_system.recursive_create_directory (file_system.dirname (a_name))
			end
			indentation_level := 0
			create open_tags.make
		end

feature -- Access

	indentation_level: INTEGER
			-- Current indentaiton level

	open_tags: DS_LINKED_STACK [STRING]
			-- Open HTML tags

feature -- HTML Output

	print_html_head (a_title: STRING) is
			-- Print an HTML Head tag with 'a_title' as title.
		require
			a_title_not_void: a_title /= Void
		do
			start_tag ("head")
				tag_content ("title", a_title+" ("+context.short_documentation_title+")")
				tag_attributes ("link", << ["rel", "stylesheet"], ["type", "text/css"], ["href", context.relative_output_directory+"style.css"] >>)
			end_tag
		end

feature -- Basic operations

	indent is
			-- Indent.
		do
			put_string (create {STRING}.make_filled (' ', indentation_level*2))
		end

	start_tag (a_tag: STRING) is
			-- Start new tag `a_tag'.
		do
			start_tag_attributes (a_tag, << >>)
		end

	start_tag_css (a_tag, a_css_class: STRING) is
			-- Start new tag `a_tag' with css attribute `a_css_class'.
		do
			start_tag_attributes (a_tag, << ["class", a_css_class] >>)
		end

	start_tag_attributes (a_tag: STRING; an_attributes: ARRAY [TUPLE [STRING, STRING]]) is
			-- Start new tag `a_tag' with attributes `an_attributes'.
		do
			indent
			indentation_level := indentation_level + 1
			put_string ("<"+a_tag+attributes_string (an_attributes)+">%N")
			open_tags.put (a_tag)
		end

	content_line (a_line: STRING) is
			-- Content line between start and end tag.
		do
			indent
			put_string (a_line+"%N")
		end

	end_tag is
			-- End tag
		do
			indentation_level := indentation_level - 1
			indent
			put_string ("</"+open_tags.item+">%N")
			open_tags.remove
		end

	tag (a_tag: STRING) is
			-- Single tag.
		do
			tag_attributes (a_tag, << >>)
		end

	tag_attributes (a_tag: STRING; an_attributes: ARRAY [TUPLE [STRING, STRING]]) is
			-- Single tag with attributes.
		do
			indent
			put_string ("<"+a_tag+attributes_string (an_attributes)+" />%N")
		end

	tag_content (a_tag, a_content: STRING) is
			-- Singe tag with content.
		do
			tag_attributes_content (a_tag, << >>, a_content)
		end

	tag_css_content (a_tag, a_css_class, a_content: STRING) is
			-- Single tag with a css class and content.
		do
			tag_attributes_content (a_tag, << ["class", a_css_class] >>, a_content)
		end

	tag_attributes_content (a_tag: STRING; an_attributes: ARRAY [TUPLE [STRING, STRING]]; a_content: STRING) is
			-- Single tag with attributes and content.
		require
			open_write: is_open_write
		do
			indent
			put_string ("<"+a_tag+attributes_string (an_attributes)+">"+a_content+"</"+a_tag+">%N")
		end

	tag_content_to_string (a_tag, a_content: STRING): STRING is
			-- Singe tag with content.
		do
			Result := tag_attributes_content_to_string (a_tag, << >>, a_content)
		end

	tag_css_content_to_string (a_tag, a_css_class, a_content: STRING): STRING is
			-- Single tag with a css class and content.
		do
			Result := tag_attributes_content_to_string (a_tag, << ["class", a_css_class] >>, a_content)
		end

	tag_attributes_content_to_string (a_tag: STRING; an_attributes: ARRAY [TUPLE [STRING, STRING]]; a_content: STRING): STRING is
			-- Single tag with attributes and content.
		require
			open_write: is_open_write
		do
			Result := "<"+a_tag+attributes_string (an_attributes)+">"+a_content+"</"+a_tag+">"
		end

	link_content (a_link, a_content: STRING) is
			-- Print a link to 'a_link' with 'a_content'.
		do
			indent
			put_string ("<a href=%""+a_link+"%">"+a_content+"</a>%N")
		end

	link_css_content (a_link, a_css_class, a_content: STRING) is
			-- Print a link to 'a_link' with 'a_css_class' and 'a_content'.
		do
			indent
			put_string ("<a href=%""+a_link+"%" class=%""+a_css_class+"%">"+a_content+"</a>%N")
		end

	link_content_to_string (a_link, a_content: STRING): STRING is
			-- Print a link to 'a_link' with 'a_content' to a string.
		do
			Result := "<a href=%""+a_link+"%">"+a_content+"</a>"
		end

	link_css_content_to_string (a_link, a_css_class, a_content: STRING): STRING is
			-- Print a link to 'a_link' with 'a_css_class' and 'a_content' to a string.
		do
			Result := "<a href=%""+a_link+"%" class=%""+a_css_class+"%">"+a_content+"</a>"
		end

	anchor_content (a_name, a_content: STRING) is
			-- Print an anchor with 'a_name' and 'a_content' to a string.
		do
			indent
			put_string (anchor_content_to_string (a_name, a_content)+"%N")
		end

	anchor_content_to_string (a_name, a_content: STRING): STRING is
			-- Print an anchor with 'a_name' and 'a_content' to a string.
		do
			Result := "<a name=%""+a_name+"%">"+a_content+"</a>"
		end

	print_overview_buttons is
			-- Print buttons panel for overview page
		do
			start_tag_css ("div", css_buttons)
				tag_css_content ("span", css_active_button, "Overview")
				tag_css_content ("span", css_button, "Cluster")
				tag_css_content ("span", css_button, "Class")
--				link_css_content (context.relative_clusters_link, css_button, "Clusters")
				link_css_content (context.relative_classes_link, css_button, "Classes")
				link_css_content (context.relative_index_link ('a'), css_button, "Index")
				tag_content ("span", "&nbsp;&nbsp;&nbsp;&nbsp;")
				tag_css_content ("span", css_button, "Previous")
				tag_css_content ("span", css_button, "Next")
			end_tag
		end

	print_cluster_file_buttons (a_cluster: ET_CLUSTER) is
			-- Print buttons panel for cluster file page
		do
			start_tag_css ("div", css_buttons)
				link_css_content (context.relative_overview_link, css_button, "Overview")
				tag_css_content ("span", css_active_button, "Cluster")
				tag_css_content ("span", css_button, "Class")
--				link_css_content (context.relative_clusters_link, css_button, "Clusters")
				link_css_content (context.relative_classes_link, css_button, "Classes")
				link_css_content (context.relative_index_link (a_cluster.name.item (1)), css_button, "Index")
				tag_content ("span", "&nbsp;&nbsp;&nbsp;&nbsp;")
				if context.previous_link = Void then
					tag_css_content ("span", css_button, "Previous")
				else
					link_css_content (context.previous_link, css_button, "Previous")
				end
				if context.next_link = Void then
					tag_css_content ("span", css_button, "Next")
				else
					link_css_content (context.next_link, css_button, "Next")
				end
			end_tag
		end

	print_class_file_buttons (a_class: ET_CLASS) is
			-- Print buttons panel for class file page
		do
			start_tag_css ("div", css_buttons)
				link_css_content (context.relative_overview_link, css_button, "Overview")
				link_css_content (context.cluster_link_by_cluster (a_class.cluster), css_button, "Cluster")
				tag_css_content ("span", css_active_button, "Class")
--				link_css_content (context.relative_clusters_link, css_button, "Clusters")
				link_css_content (context.relative_classes_link, css_button, "Classes")
				link_css_content (context.relative_index_link (a_class.name.name.item (1)), css_button, "Index")
				tag_content ("span", "&nbsp;&nbsp;&nbsp;&nbsp;")
				if context.previous_link = Void then
					tag_css_content ("span", css_button, "Previous")
				else
					link_css_content (context.previous_link, css_button, "Previous")
				end
				if context.next_link = Void then
					tag_css_content ("span", css_button, "Next")
				else
					link_css_content (context.next_link, css_button, "Next")
				end
				tag_content ("span", "&nbsp;&nbsp;&nbsp;&nbsp;")
				link_css_content ("#top", css_button, "Top")
				link_css_content ("#features", css_button, "Features")
			end_tag
		end

	print_index_buttons is
			-- Print buttons panel for index page
		do
			start_tag_css ("div", css_buttons)
				link_css_content (context.relative_overview_link, css_button, "Overview")
				tag_css_content ("span", css_button, "Cluster")
				tag_css_content ("span", css_button, "Class")
--				link_css_content (context.relative_clusters_link, css_button, "Clusters")
				link_css_content (context.relative_classes_link, css_button, "Classes")
				tag_css_content ("span", css_active_button, "Index")
				tag_content ("span", "&nbsp;&nbsp;&nbsp;&nbsp;")
				if context.previous_link = Void then
					tag_css_content ("span", css_button, "Previous")
				else
					link_css_content (context.previous_link, css_button, "Previous")
				end
				if context.next_link = Void then
					tag_css_content ("span", css_button, "Next")
				else
					link_css_content (context.next_link, css_button, "Next")
				end
			end_tag
		end

	print_classes_buttons is
			-- Print buttons panel for classes page
		do
			start_tag_css ("div", css_buttons)
				link_css_content (context.relative_overview_link, css_button, "Overview")
				tag_css_content ("span", css_button, "Cluster")
				tag_css_content ("span", css_button, "Class")
--				link_css_content (context.relative_clusters_link, css_button, "Clusters")
				tag_css_content ("span", css_active_button, "Classes")
				link_css_content (context.relative_index_link ('a'), css_button, "Index")
				tag_content ("span", "&nbsp;&nbsp;&nbsp;&nbsp;")
				tag_css_content ("span", css_button, "Previous")
				tag_css_content ("span", css_button, "Next")
			end_tag
		end

	print_clusters_buttons is
			-- Print buttons panel for clusters page
		do
			start_tag_css ("div", css_buttons)
				link_css_content (context.relative_overview_link, css_button, "Overview")
				tag_css_content ("span", css_button, "Cluster")
				tag_css_content ("span", css_button, "Class")
				tag_css_content ("span", css_active_button, "Clusters")
				link_css_content (context.relative_classes_link, css_button, "Classes")
				link_css_content (context.relative_index_link ('a'), css_button, "Index")
				tag_content ("span", "&nbsp;&nbsp;&nbsp;&nbsp;")
				tag_css_content ("span", css_button, "Previous")
				tag_css_content ("span", css_button, "Next")
			end_tag
		end

	print_header_start is
			-- Start with the header.
			-- Call `print_header_end' to finish the header.
		do
			start_tag_css ("div", css_header)
			tag_attributes_content ("div", << ["class", css_header_title], ["id", "top"] >> , context.short_documentation_title)
		end

	print_header_end is
			-- End the header.
			-- Call `print_header_start' before.
		do
			end_tag
			tag ("hr")
		end

	print_footer_start is
			-- Start with the footer.
			-- Call `print_footer_end' to finish the footer.
		do
			tag ("hr")
			start_tag ("p")
			tag ("br")
			end_tag
			start_tag_css ("div", css_footer)
			tag_css_content ("div", css_footer_title, context.short_documentation_title)
		end

	print_footer_end is
			-- End the footer.
			-- Call `print_footer_start' before.
		do
			end_tag
		end

feature -- Conveniance printing

	print_cluster_name (a_cluster: ET_CLUSTER) is
			-- Print class name of 'a_cluster' to string.
		do
			put_string (print_cluster_name_to_string (a_cluster))
		end

	print_cluster_name_to_string (a_cluster: ET_CLUSTER): STRING is
			-- Print cluster name of 'a_cluster' to string.
		require
			a_cluster_not_void: a_cluster /= Void
		local
			a_link: STRING
		do
			a_link := context.cluster_link_by_cluster (a_cluster)
			if a_link = Void then
				Result := tag_css_content_to_string ("span", css_cluster_name, a_cluster.name)
			else
				Result := link_css_content_to_string (a_link, css_linked_cluster_name, a_cluster.name)
			end
		ensure
			result_exists: Result /= Void
		end

	print_qualified_cluster_name_single_link (a_cluster: ET_CLUSTER) is
			-- Print qualified cluster name of 'a_cluster' as a link to 'a_cluster'.
		require
			a_cluster_not_void: a_cluster /= Void
		local
			a_parent: ET_CLUSTER
			a_content: STRING
			a_link: STRING
		do
			create a_content.make_from_string (a_cluster.name)
			from
				a_parent := a_cluster.parent
			until
				a_parent = Void
			loop
				a_content.prepend_string (a_parent.name+".")
				a_parent := a_parent.parent
			end
			indent
			a_link := context.cluster_link_by_cluster (a_cluster)
			if a_link = Void then
				tag_css_content ("span", css_cluster_name, a_content)
			else
				link_css_content (a_link, css_cluster_name, a_content)
			end
		end

	print_qualified_cluster_name_to_string (a_cluster: ET_CLUSTER; link_a_cluster: BOOLEAN): STRING is
			-- Print cluster name of 'a_cluster' and all its parents to string.
		require
			a_cluster_not_void: a_cluster /= Void
		local
			a_parent: ET_CLUSTER
		do
			if link_a_cluster then
				create Result.make_from_string (print_cluster_name_to_string (a_cluster))
			else
				create Result.make_from_string (tag_css_content_to_string ("span", css_cluster_name, a_cluster.name))
			end
			from
				a_parent := a_cluster.parent
			until
				a_parent = Void
			loop
				Result.prepend_string (print_cluster_name_to_string (a_parent)+".")
				a_parent := a_parent.parent
			end
		ensure
			result_exists: Result /= Void
		end

	print_class_name (a_class: ET_CLASS) is
			-- Print class name of 'a_class'.
		do
			put_string (print_class_name_to_string (a_class))
		end

	print_class_name_to_string (a_class: ET_CLASS): STRING is
			-- Print class name of 'a_class' to string.
		require
			a_class_not_void: a_class /= Void
		local
			a_link: STRING
		do
			a_link := context.class_link_by_class (a_class)
			if a_link = Void then
				Result := tag_css_content_to_string ("span", css_class_name, a_class.name.name)
			else
				Result := link_css_content_to_string (a_link, css_linked_class_name, a_class.name.name)
			end
		ensure
			result_exists: Result /= Void
		end

	print_qualified_class_name (a_class: ET_CLASS) is
			-- Print qualified class name of 'a_class'.
		do
			put_string (print_qualified_class_name_to_string (a_class))
		end

	print_qualified_class_name_to_string (a_class: ET_CLASS): STRING is
			-- Print qualified class name of 'a_class' to string.
		do
			Result := print_qualified_cluster_name_to_string (a_class.cluster, True)+"."+print_class_name_to_string (a_class)
		end

	print_subclusters_tree (a_cluster: ET_CLUSTER) is
			-- Print tree of subclusters of 'a_cluster'.
		require
			a_cluster_not_void: a_cluster /= Void
		local
			cluster_cursor: DS_LIST_CURSOR [ET_CLUSTER]
			a_item: ET_CLUSTER
		do
			if a_cluster.subclusters /= Void then
				a_cluster.subclusters.clusters.sort (create {DS_QUICK_SORTER [ET_CLUSTER]}.make (create {CLUSTER_COMPARATOR}))
--				start_tag_css ("ul", css_cluster_tree)
				cluster_cursor := a_cluster.subclusters.clusters.new_cursor
				from
					cluster_cursor.start
				until
					cluster_cursor.after
				loop
					a_item := cluster_cursor.item
					if not context.ignored_clusters.has (a_item.name) then
						start_tag ("li")
						print_qualified_cluster_name_single_link (a_item)
						end_tag
						print_subclusters_tree (a_item)
					end
					cluster_cursor.forth
				end
--				end_tag
			end
		end

feature {NONE} -- Implementation

	attributes_string (a_attributes: ARRAY [TUPLE [STRING, STRING]]): STRING is
			-- Start new tag `a_tag' with attributes `a_attributes'.
		local
			attribute_name, attribute_value: STRING
			i: INTEGER
		do
			create Result.make_empty
			from
				i := 1
			variant
				a_attributes.count + 1 - i
			until
				i > a_attributes.count
			loop
				attribute_name ?= a_attributes.item (i).item (1)
				attribute_value ?= a_attributes.item (i).item (2)
				Result.append_string (" "+attribute_name+"=%""+attribute_value+"%"")
				i := i + 1
			end
		end

end