indexing

	description: "[

		An HTML file of the index with a specific letter.

	]"
	date: "$Date$"
	revision: "$Revision$"

class EDOC_INDEX_HTML_FILE

inherit

	EDOC_HTML_OUTPUT_FILE

create

	make_index_file

feature {NONE} -- Initialisation

	make_index_file (a_letter: CHARACTER) is
			-- Initalise default values
		do
			make ("index-"+a_letter.out+".html")
			letter := a_letter
		end

feature -- Access

	letter: CHARACTER
			-- Letter of index page

feature -- Basic operations

	generate_index_file is
			-- Generate index page for `letter'.
		local
			entry_name: STRING
			index_cursor: DS_LIST_CURSOR [TUPLE [STRING, ANY]]
		do
			open_write

			put_string ("<?xml version=%"1.0%" encoding=%"ISO-8859-1%"?>%N")
			put_string ("<!DOCTYPE html PUBLIC %"-//W3C//DTD XHTML 1.1//EN%" %"http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd%">%N")
			start_tag_attributes ("html", << ["xmlns", "http://www.w3.org/1999/xhtml"] >>)

			print_html_head ("Index "+letter.as_upper.out)
			start_tag ("body")

				print_header_start
				print_index_buttons
				print_index_header
				print_header_end

				tag_content ("h1", "Index "+letter.as_upper.out)

				tag ("hr")
				start_tag_css ("ul", css_index_list)
				index_cursor := context.global_index.new_cursor
				from
					index_cursor.start
				until
					index_cursor.after
				loop
					entry_name ?= index_cursor.item.item (1)
					if entry_name.item (1) = letter then
						process_entry (index_cursor.item)
					end
					index_cursor.forth
				end
				end_tag

				print_footer_start
				print_index_buttons
				print_index_header
				print_footer_end

			end_tag
			end_tag

			close
		end

	process_entry (an_entry: TUPLE [STRING, ANY]) is
			-- Porcess 'an_entry'.
		local
			a_cluster: ET_CLUSTER
			a_class: ET_CLASS
			a_feature: ET_FEATURE
			a_line: STRING
		do
			a_cluster ?= an_entry.item (2)
			a_class ?= an_entry.item (2)
			a_feature ?= an_entry.item (2)
			start_tag ("li")
			if a_cluster /= Void then
				a_line := print_cluster_name_to_string (a_cluster)
				if a_cluster.parent /= Void then
					a_line.append_string (tag_css_content_to_string ("span", css_index_small, " - Cluster in "+print_qualified_cluster_name_to_string (a_cluster.parent, True)))
				else
					a_line.append_string (tag_css_content_to_string ("span", css_index_small, " - Cluster"))
				end
				content_line (a_line)
			elseif a_class /= Void then
				a_line := print_class_name_to_string (a_class)
				a_line.append_string (tag_css_content_to_string ("span", css_index_small, " - Class in "+print_qualified_cluster_name_to_string (a_class.cluster, True)))
				content_line (a_line)
			elseif a_feature /= Void then
				a_line := link_css_content_to_string (context.feature_link_by_feature (a_feature, a_feature.implementation_class), css_linked_feature_name, context.escaped_feature_name (a_feature))
				a_line.append_string (tag_css_content_to_string ("span", css_index_small, " - Feature in "+print_qualified_class_name_to_string (a_feature.implementation_class)))
				content_line (a_line)
			end
			end_tag
		end

	print_index_header is
			-- Print buttons for index.
		local
			a_letter: CHARACTER
		do
			start_tag ("div")
			from
				a_letter := 'a'
			until
				a_letter > 'z'
			loop
				if a_letter = letter then
					tag_css_content ("span", css_active_button, a_letter.as_upper.out)
				else
					link_css_content ("index-"+a_letter.out+".html", css_button, a_letter.as_upper.out)
				end
				a_letter := a_letter.next
			end
			end_tag
		end

end